# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class OperationResult(object):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None):
        self.result_status = result_status

        if result_reason is not None:
            self.result_reason = result_reason
        else:
            self.result_reason = None

        if result_message is not None:
            self.result_message = result_message
        else:
            self.result_message = None


class CreateResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 object_type=None,
                 uuid=None,
                 template_attribute=None):
        super(CreateResult, self).__init__(
            result_status, result_reason, result_message)
        if object_type is not None:
            self.object_type = object_type
        else:
            self.object_type = None

        if uuid is not None:
            self.uuid = uuid
        else:
            self.uuid = None

        if template_attribute is not None:
            self.template_attribute = template_attribute
        else:
            self.template_attribute = None


class CreateKeyPairResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 private_key_uuid=None,
                 public_key_uuid=None,
                 private_key_template_attribute=None,
                 public_key_template_attribute=None):
        super(CreateKeyPairResult, self).__init__(
            result_status, result_reason, result_message)
        self.private_key_uuid = private_key_uuid
        self.public_key_uuid = public_key_uuid
        self.private_key_template_attribute = private_key_template_attribute
        self.public_key_template_attribute = public_key_template_attribute


class ActivateResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 uuid=None):
        super(ActivateResult, self).__init__(
            result_status, result_reason, result_message)

        if uuid is not None:
            self.uuid = uuid
        else:
            self.uuid = None


class RegisterResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 uuid=None,
                 template_attribute=None):
        super(RegisterResult, self).__init__(
            result_status, result_reason, result_message)
        if uuid is not None:
            self.uuid = uuid
        else:
            self.uuid = None

        if template_attribute is not None:
            self.template_attribute = template_attribute
        else:
            self.template_attribute = None


class RekeyKeyPairResult(CreateKeyPairResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 private_key_uuid=None,
                 public_key_uuid=None,
                 private_key_template_attribute=None,
                 public_key_template_attribute=None):
        super(RekeyKeyPairResult, self).__init__(
            result_status, result_reason, result_message, private_key_uuid,
            public_key_uuid, private_key_template_attribute,
            public_key_template_attribute)


class GetResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 object_type=None,
                 uuid=None,
                 secret=None):
        super(GetResult, self).__init__(
            result_status, result_reason, result_message)
        if object_type is not None:
            self.object_type = object_type
        else:
            self.object_type = None

        if uuid is not None:
            self.uuid = uuid
        else:
            self.uuid = None

        if secret is not None:
            self.secret = secret
        else:
            self.secret = None


class GetAttributesResult(OperationResult):

    def __init__(
            self,
            result_status,
            result_reason=None,
            result_message=None,
            uuid=None,
            attributes=None
    ):
        super(GetAttributesResult, self).__init__(
            result_status,
            result_reason,
            result_message
        )
        self.uuid = uuid
        self.attributes = attributes


class GetAttributeListResult(OperationResult):

    def __init__(
            self,
            result_status,
            result_reason=None,
            result_message=None,
            uid=None,
            names=None):
        super(GetAttributeListResult, self).__init__(
            result_status, result_reason, result_message)
        self.uid = uid
        self.names = names


class DestroyResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 uuid=None):
        super(DestroyResult, self).__init__(
            result_status, result_reason, result_message)
        if uuid is not None:
            self.uuid = uuid
        else:
            self.uuid = None


class LocateResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 uuids=None):
        super(LocateResult, self).__init__(
            result_status, result_reason, result_message)
        self.uuids = uuids


class QueryResult(OperationResult):
    """
    A container for the results of a Query operation.

    Attributes:
        result_status: The status of the Query operation (e.g., success or
            failure).
        result_reason: The reason for the operation status.
        result_message: Extra information pertaining to the status reason.
        operations: A list of Operations supported by the server.
        object_types: A list of Object Types supported by the server.
        vendor_identification:
        server_information:
        application_namespaces: A list of namespaces supported by the server.
        extension_information: A list of extensions supported by the server.
    """

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 operations=None,
                 object_types=None,
                 vendor_identification=None,
                 server_information=None,
                 application_namespaces=None,
                 extension_information=None):
        super(QueryResult, self).__init__(
            result_status, result_reason, result_message)

        if operations is None:
            self.operations = list()
        else:
            self.operations = operations

        if object_types is None:
            self.object_types = list()
        else:
            self.object_types = object_types

        self.vendor_identification = vendor_identification
        self.server_information = server_information

        if application_namespaces is None:
            self.application_namespaces = list()
        else:
            self.application_namespaces = application_namespaces

        if extension_information is None:
            self.extension_information = list()
        else:
            self.extension_information = extension_information


class DiscoverVersionsResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 protocol_versions=None):
        super(DiscoverVersionsResult, self).__init__(
            result_status, result_reason, result_message)
        self.protocol_versions = protocol_versions


class RevokeResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 unique_identifier=None):
        super(RevokeResult, self).__init__(
            result_status, result_reason, result_message)
        self.unique_identifier = unique_identifier


class MACResult(OperationResult):

    def __init__(self,
                 result_status,
                 result_reason=None,
                 result_message=None,
                 uuid=None,
                 mac_data=None):
        super(MACResult, self).__init__(
            result_status,
            result_reason,
            result_message
        )
        self.uuid = uuid
        self.mac_data = mac_data



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import TestCase

from kmip.core.attributes import CertificateType
from kmip.core.attributes import CryptographicParameters
from kmip.core.attributes import DerivationParameters
from kmip.core.attributes import DigestValue
from kmip.core.attributes import HashingAlgorithm
from kmip.core.attributes import Name
from kmip.core.attributes import OperationPolicyName

from kmip.core import enums
from kmip.core.enums import HashingAlgorithm as HashingAlgorithmEnum
from kmip.core.enums import NameType

from kmip.core.utils import BytearrayStream


class TestNameValue(TestCase):

    def setUp(self):
        super(TestNameValue, self).setUp()
        self.stream = BytearrayStream()
        self.stringName1 = 'Jenny'
        self.stringName2 = 'Johnny'

    def tearDown(self):
        super(TestNameValue, self).tearDown()

    def test_write_no_padding(self):
        self.skipTest('Not implemented')

    def test_write_with_padding(self):
        self.skipTest('Not implemented')

    def test_read_no_padding(self):
        self.skipTest('Not implemented')

    def test_read_with_padding(self):
        self.skipTest('Not implemented')

    def test__eq(self):
        name_val = Name.NameValue(self.stringName1)
        same_name_val = Name.NameValue(self.stringName1)
        other_name_val = Name.NameValue(self.stringName2)

        self.assertTrue(name_val == same_name_val)
        self.assertFalse(name_val == other_name_val)
        self.assertFalse(name_val == 'invalid')

    def test__ne(self):
        name_val = Name.NameValue(self.stringName1)
        other_name_val = Name.NameValue(self.stringName2)

        self.assertTrue(name_val != other_name_val)
        self.assertTrue(name_val != 'invalid')

    def test__str(self):
        name_val = Name.NameValue(self.stringName1)
        repr_name = "NameValue(value='{0}')".format(self.stringName1)

        self.assertEqual(self.stringName1, str(name_val))
        self.assertEqual(repr_name, repr(name_val))


class TestNameType(TestCase):

    def setUp(self):
        super(TestNameType, self).setUp()
        self.stream = BytearrayStream()
        self.enum_uri = NameType.URI
        self.enum_txt = NameType.UNINTERPRETED_TEXT_STRING

    def tearDown(self):
        super(TestNameType, self).tearDown()

    def test_write_no_padding(self):
        self.skipTest('Not implemented')

    def test_write_with_padding(self):
        self.skipTest('Not implemented')

    def test_read_no_padding(self):
        self.skipTest('Not implemented')

    def test_read_with_padding(self):
        self.skipTest('Not implemented')

    def test__eq(self):
        type_uri = Name.NameType(self.enum_uri)
        same_type = Name.NameType(self.enum_uri)
        type_txt = Name.NameType(self.enum_txt)

        self.assertTrue(type_uri == same_type)
        self.assertFalse(type_uri == type_txt)
        self.assertFalse(type_uri == 'invalid')

    def test__ne(self):
        type_uri = Name.NameType(self.enum_uri)
        same_type = Name.NameType(self.enum_uri)
        type_txt = Name.NameType(self.enum_txt)

        self.assertFalse(type_uri != same_type)
        self.assertTrue(type_uri != type_txt)
        self.assertTrue(type_uri != 'invalid')

    def test__str(self):
        type_uri = Name.NameType(self.enum_uri)
        str_uri = "{0}".format(self.enum_uri)
        repr_uri = "NameType(value=<{0}: {1}>)".format(
                self.enum_uri,
                self.enum_uri.value)

        self.assertEqual(str_uri, str(type_uri))
        self.assertEqual(repr_uri, repr(type_uri))


class TestName(TestCase):

    def setUp(self):
        super(TestName, self).setUp()
        self.stream = BytearrayStream()
        self.badFormatName = 8675309
        self.stringName1 = 'Jenny'
        self.stringName2 = 'Johnny'
        self.enumNameType = NameType.UNINTERPRETED_TEXT_STRING
        self.enumNameTypeUri = NameType.URI

    def tearDown(self):
        super(TestName, self).tearDown()

    def test_bad_name_value_format(self):
        """
         Test that an error is raised in for an incorrectly formatted name
         value
        """
        name_obj = Name()
        name_obj.name_value = self.badFormatName
        name_obj.name_type = self.enumNameType

        self.assertRaises(TypeError, name_obj.validate)

    def test_bad_name_type_format(self):
        """
         Test that an error is raised for an incorrectly formatted name type
        """
        name_obj = Name()
        name_obj.name_value = self.stringName1
        name_obj.name_type = self.badFormatName

        self.assertRaises(TypeError, name_obj.validate)

    def test_name_create_string_input(self):
        """
         Test the creation of object names with an enum value for the name type
        """
        name_obj = Name.create(self.stringName1, self.enumNameType)
        self.assertIsInstance(name_obj.name_value, Name.NameValue)
        self.assertEqual(self.stringName1, name_obj.name_value.value)

    def test_name_create_bad_input(self):
        """
         Test the creation of object names with a bad value input
        """
        name_value = self.badFormatName
        name_type = self.enumNameType

        self.assertRaises(TypeError, Name.create, *(name_value, name_type))

    def test_name_create_bad_type_input(self):
        """
         Test the creation of object names with a bad value input
        """
        self.assertRaises(TypeError, Name.create, *(self.stringName1,
                                                    self.badFormatName))

    def test__eq(self):
        name_obj = Name.create(self.stringName1, self.enumNameType)
        same_name = Name.create(self.stringName1, self.enumNameType)
        other_name = Name.create(self.stringName2, self.enumNameType)
        other_type = Name.create(self.stringName1, self.enumNameTypeUri)

        self.assertTrue(name_obj == same_name)
        self.assertFalse(name_obj == other_name)
        self.assertFalse(name_obj == other_type)
        self.assertFalse(name_obj == 'invalid')

    def test__ne(self):
        name_obj = Name.create(self.stringName1, self.enumNameType)
        same_name = Name.create(self.stringName1, self.enumNameType)
        other_name = Name.create(self.stringName2, self.enumNameType)
        other_type = Name.create(self.stringName1, self.enumNameTypeUri)

        self.assertFalse(name_obj != same_name)
        self.assertNotEqual(name_obj, other_name)
        self.assertNotEqual(name_obj, other_type)

    def test__str(self):
        name_obj = Name.create(self.stringName1, self.enumNameType)
        repr_name = (
                "Name(type=NameType(value="
                "<NameType.UNINTERPRETED_TEXT_STRING: {0}>),"
                "value=NameValue(value='{1}'))"
                ).format(self.enumNameType.value, self.stringName1)

        self.assertEqual(self.stringName1, str(name_obj))
        self.assertEqual(repr_name, repr(name_obj))


class TestOperationPolicyName(TestCase):

    def setUp(self):
        super(TestOperationPolicyName, self).setUp()

    def tearDown(self):
        super(TestOperationPolicyName, self).tearDown()

    def _test_operation_policy_name(self, value):
        opn = OperationPolicyName(value)

        if value is None:
            value = ''

        msg = "expected {0}, received {1}".format(value, opn.value)
        self.assertEqual(value, opn.value, msg)

    def test_operation_policy_name(self):
        self._test_operation_policy_name('test')

    def test_operation_policy_name_on_none(self):
        self._test_operation_policy_name(None)


class TestHashingAlgorithm(TestCase):
    """
    A test suite for the HashingAlgorithm class.

    Since HashingAlgorithm is a simple wrapper for the Enumeration primitive,
    only a few tests pertaining to construction are needed.
    """

    def setUp(self):
        super(TestHashingAlgorithm, self).setUp()

    def tearDown(self):
        super(TestHashingAlgorithm, self).tearDown()

    def _test_init(self, value):
        if (isinstance(value, HashingAlgorithmEnum)) or (value is None):
            hashing_algorithm = HashingAlgorithm(value)

            msg = "expected {0}, observed {1}".format(
                value, hashing_algorithm.value)
            self.assertEqual(value, hashing_algorithm.value, msg)
        else:
            self.assertRaises(TypeError, HashingAlgorithm, value)

    def test_init_with_none(self):
        """
        Test that a HashingAlgorithm object can be constructed with no
        specified value.
        """
        self._test_init(None)

    def test_init_with_valid(self):
        """
        Test that a HashingAlgorithm object can be constructed with a valid
        HashingAlgorithm enumeration value.
        """
        self._test_init(HashingAlgorithmEnum.MD5)

    def test_init_with_invalid(self):
        """
        Test that a TypeError exception is raised when a non HashingAlgorithm
        enumeration value is used to construct a HashingAlgorithm object.
        """
        self._test_init("invalid")


# TODO (peter-hamilton) Replace with generic Enumeration subclass test suite.
class TestCertificateType(TestCase):
    """
    A test suite for the CertificateType class.

    Since CertificateType is a simple wrapper for the Enumeration primitive,
    only a few tests pertaining to construction are needed.
    """

    def setUp(self):
        super(TestCertificateType, self).setUp()

    def tearDown(self):
        super(TestCertificateType, self).tearDown()

    def _test_init(self, value):
        if (isinstance(value, enums.CertificateType)) or (value is None):
            if value is None:
                certificate_type = CertificateType()
                value = enums.CertificateType.X_509
            else:
                certificate_type = CertificateType(value)

            msg = "expected {0}, observed {1}".format(
                value, certificate_type.value)
            self.assertEqual(value, certificate_type.value, msg)
        else:
            self.assertRaises(TypeError, CertificateType, value)

    def test_init_with_none(self):
        """
        Test that a CertificateType object can be constructed with no specified
        value.
        """
        self._test_init(None)

    def test_init_with_valid(self):
        """
        Test that a CertificateType object can be constructed with valid byte
        data.
        """
        self._test_init(enums.CertificateType.PGP)


class TestDigestValue(TestCase):
    """
    A test suite for the DigestValue class.

    Since DigestValue is a simple wrapper for the ByteString primitive, only
    a few tests pertaining to construction are needed.
    """

    def setUp(self):
        super(TestDigestValue, self).setUp()

    def tearDown(self):
        super(TestDigestValue, self).tearDown()

    def _test_init(self, value):
        if (isinstance(value, bytes)) or (value is None):
            digest_value = DigestValue(value)

            if value is None:
                value = bytes()

            msg = "expected {0}, observed {1}".format(
                value, digest_value.value)
            self.assertEqual(value, digest_value.value, msg)
        else:
            self.assertRaises(TypeError, DigestValue, value)

    def test_init_with_none(self):
        """
        Test that a DigestValue object can be constructed with no specified
        value.
        """
        self._test_init(None)

    def test_init_with_valid(self):
        """
        Test that a DigestValue object can be constructed with valid byte data.
        """
        self._test_init(b'\x00\x01\x02\x03')


class TestCryptographicParameters(TestCase):
    """
    Test suite for the CryptographicParameters struct.
    """

    def setUp(self):
        super(TestCryptographicParameters, self).setUp()

        # Encoding obtained in part from the KMIP 1.1 testing document,
        # Section 11.1. The rest of the encoding for KMIP 1.2+ features was
        # built by hand; later KMIP testing documents do not include the
        # encoding, so a manual construction is necessary.
        #
        # This encoding matches the following set of values:
        # Block Cipher Mode - CBC
        # Padding Method - PKCS5
        # Hashing Algorithm - SHA-1
        # Key Role Type - KEK
        # Digital Signature Algorithm - SHA-256 with RSA
        # Cryptographic Algorithm - AES
        # Random IV - True
        # IV Length - 96
        # Tag Length - 128
        # Fixed Field Length - 32
        # Invocation Field Length - 64
        # Counter Length - 0
        # Initial Counter Value - 1

        self.full_encoding = BytearrayStream(
            b'\x42\x00\x2B\x01\x00\x00\x00\xD0'
            b'\x42\x00\x11\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x5F\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\x38\x05\x00\x00\x00\x04\x00\x00\x00\x04\x00\x00\x00\x00'
            b'\x42\x00\x83\x05\x00\x00\x00\x04\x00\x00\x00\x0B\x00\x00\x00\x00'
            b'\x42\x00\xAE\x05\x00\x00\x00\x04\x00\x00\x00\x05\x00\x00\x00\x00'
            b'\x42\x00\x28\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\xC5\x06\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00\x01'
            b'\x42\x00\xCD\x02\x00\x00\x00\x04\x00\x00\x00\x60\x00\x00\x00\x00'
            b'\x42\x00\xCE\x02\x00\x00\x00\x04\x00\x00\x00\x80\x00\x00\x00\x00'
            b'\x42\x00\xCF\x02\x00\x00\x00\x04\x00\x00\x00\x20\x00\x00\x00\x00'
            b'\x42\x00\xD2\x02\x00\x00\x00\x04\x00\x00\x00\x40\x00\x00\x00\x00'
            b'\x42\x00\xD0\x02\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x42\x00\xD1\x02\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

        # Adapted from the full encoding above. This encoding matches the
        # following set of values:
        # Block Cipher Mode - CBC
        # Padding Method - PKCS5
        # Hashing Algorithm - SHA-1
        # Key Role Type - KEK
        # Digital Signature Algorithm - SHA-256 with RSA
        # Cryptographic Algorithm - AES
        # Tag Length - 128
        # Initial Counter Value - 1

        self.partial_encoding = BytearrayStream(
            b'\x42\x00\x2B\x01\x00\x00\x00\x80'
            b'\x42\x00\x11\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x5F\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\x38\x05\x00\x00\x00\x04\x00\x00\x00\x04\x00\x00\x00\x00'
            b'\x42\x00\x83\x05\x00\x00\x00\x04\x00\x00\x00\x0B\x00\x00\x00\x00'
            b'\x42\x00\xAE\x05\x00\x00\x00\x04\x00\x00\x00\x05\x00\x00\x00\x00'
            b'\x42\x00\x28\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\xCE\x02\x00\x00\x00\x04\x00\x00\x00\x80\x00\x00\x00\x00'
            b'\x42\x00\xD1\x02\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

        self.empty_encoding = BytearrayStream(
            b'\x42\x00\x2B\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestCryptographicParameters, self).tearDown()

    def test_init(self):
        """
        Test that a CryptographicParameters struct can be constructed with
        no arguments.
        """
        cryptographic_parameters = CryptographicParameters()

        self.assertEqual(None, cryptographic_parameters.block_cipher_mode)
        self.assertEqual(None, cryptographic_parameters.padding_method)
        self.assertEqual(None, cryptographic_parameters.hashing_algorithm)
        self.assertEqual(None, cryptographic_parameters.key_role_type)
        self.assertEqual(
            None,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            None,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(None, cryptographic_parameters.random_iv)
        self.assertEqual(None, cryptographic_parameters.iv_length)
        self.assertEqual(None, cryptographic_parameters.tag_length)
        self.assertEqual(None, cryptographic_parameters.fixed_field_length)
        self.assertEqual(
            None,
            cryptographic_parameters.invocation_field_length
        )
        self.assertEqual(None, cryptographic_parameters.counter_length)
        self.assertEqual(None, cryptographic_parameters.initial_counter_value)

    def test_init_with_args(self):
        """
        Test that a CryptographicParameters struct can be constructed with
        valid values.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CTR,
            padding_method=enums.PaddingMethod.NONE,
            hashing_algorithm=enums.HashingAlgorithm.SHA_256,
            key_role_type=enums.KeyRoleType.BDK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA1_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.TRIPLE_DES,
            random_iv=False,
            iv_length=128,
            tag_length=256,
            fixed_field_length=48,
            invocation_field_length=60,
            counter_length=20,
            initial_counter_value=2
        )

        self.assertEqual(
            enums.BlockCipherMode.CTR,
            cryptographic_parameters.block_cipher_mode
        )
        self.assertEqual(
            enums.PaddingMethod.NONE,
            cryptographic_parameters.padding_method
        )
        self.assertEqual(
            enums.HashingAlgorithm.SHA_256,
            cryptographic_parameters.hashing_algorithm
        )
        self.assertEqual(
            enums.KeyRoleType.BDK,
            cryptographic_parameters.key_role_type
        )
        self.assertEqual(
            enums.DigitalSignatureAlgorithm.SHA1_WITH_RSA_ENCRYPTION,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            enums.CryptographicAlgorithm.TRIPLE_DES,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(False, cryptographic_parameters.random_iv)
        self.assertEqual(128, cryptographic_parameters.iv_length)
        self.assertEqual(256, cryptographic_parameters.tag_length)
        self.assertEqual(48, cryptographic_parameters.fixed_field_length)
        self.assertEqual(60, cryptographic_parameters.invocation_field_length)
        self.assertEqual(20, cryptographic_parameters.counter_length)
        self.assertEqual(2, cryptographic_parameters.initial_counter_value)

    def test_invalid_block_cipher_mode(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the block cipher mode of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'block_cipher_mode', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "block cipher mode must be a BlockCipherMode enumeration",
            setattr,
            *args
        )

    def test_invalid_padding_method(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the padding method of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'padding_method', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "padding method must be a PaddingMethod enumeration",
            setattr,
            *args
        )

    def test_invalid_hashing_algorithm(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the hashing algorithm of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'hashing_algorithm', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "hashing algorithm must be a HashingAlgorithm enumeration",
            setattr,
            *args
        )

    def test_invalid_key_role_type(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the key role type of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'key_role_type', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "key role type must be a KeyRoleType enumeration",
            setattr,
            *args
        )

    def test_invalid_digital_signature_algorithm(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the digital signature algorithm of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (
            cryptographic_parameters,
            'digital_signature_algorithm',
            'invalid'
        )
        self.assertRaisesRegex(
            TypeError,
            "digital signature algorithm must be a "
            "DigitalSignatureAlgorithm enumeration",
            setattr,
            *args
        )

    def test_invalid_cryptographic_algorithm(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the cryptographic algorithm of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'cryptographic_algorithm', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "cryptographic algorithm must be a CryptographicAlgorithm "
            "enumeration",
            setattr,
            *args
        )

    def test_invalid_random_iv(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the random IV of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'random_iv', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "random iv must be a boolean",
            setattr,
            *args
        )

    def test_invalid_iv_length(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the IV length of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'iv_length', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "iv length must be an integer",
            setattr,
            *args
        )

    def test_invalid_tag_length(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the tag length of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'tag_length', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "tag length must be an integer",
            setattr,
            *args
        )

    def test_invalid_fixed_field_length(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the fixed field length of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'fixed_field_length', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "fixed field length must be an integer",
            setattr,
            *args
        )

    def test_invalid_invocation_field_length(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the invocation field length of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'invocation_field_length', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "invocation field length must be an integer",
            setattr,
            *args
        )

    def test_invalid_counter_length(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the counter length of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'counter_length', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "counter length must be an integer",
            setattr,
            *args
        )

    def test_invalid_initial_counter_value(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the counter value of a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters()
        args = (cryptographic_parameters, 'initial_counter_value', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "initial counter value must be an integer",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a CryptographicParameters struct can be read from a data
        stream.
        """
        cryptographic_parameters = CryptographicParameters()

        self.assertEqual(None, cryptographic_parameters.block_cipher_mode)
        self.assertEqual(None, cryptographic_parameters.padding_method)
        self.assertEqual(None, cryptographic_parameters.hashing_algorithm)
        self.assertEqual(None, cryptographic_parameters.key_role_type)
        self.assertEqual(
            None,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            None,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(None, cryptographic_parameters.random_iv)
        self.assertEqual(None, cryptographic_parameters.iv_length)
        self.assertEqual(None, cryptographic_parameters.tag_length)
        self.assertEqual(None, cryptographic_parameters.fixed_field_length)
        self.assertEqual(
            None,
            cryptographic_parameters.invocation_field_length
        )
        self.assertEqual(None, cryptographic_parameters.counter_length)
        self.assertEqual(None, cryptographic_parameters.initial_counter_value)

        cryptographic_parameters.read(self.full_encoding)

        self.assertEqual(
            enums.BlockCipherMode.CBC,
            cryptographic_parameters.block_cipher_mode
        )
        self.assertEqual(
            enums.PaddingMethod.PKCS5,
            cryptographic_parameters.padding_method
        )
        self.assertEqual(
            enums.HashingAlgorithm.SHA_1,
            cryptographic_parameters.hashing_algorithm
        )
        self.assertEqual(
            enums.KeyRoleType.KEK,
            cryptographic_parameters.key_role_type
        )
        self.assertEqual(
            enums.DigitalSignatureAlgorithm.SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            enums.CryptographicAlgorithm.AES,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(True, cryptographic_parameters.random_iv)
        self.assertEqual(96, cryptographic_parameters.iv_length)
        self.assertEqual(128, cryptographic_parameters.tag_length)
        self.assertEqual(32, cryptographic_parameters.fixed_field_length)
        self.assertEqual(64, cryptographic_parameters.invocation_field_length)
        self.assertEqual(0, cryptographic_parameters.counter_length)
        self.assertEqual(1, cryptographic_parameters.initial_counter_value)

    def test_read_partial(self):
        """
        Test that a CryptographicParameters struct can be read from a partial
        data stream.
        """
        cryptographic_parameters = CryptographicParameters()

        self.assertEqual(None, cryptographic_parameters.block_cipher_mode)
        self.assertEqual(None, cryptographic_parameters.padding_method)
        self.assertEqual(None, cryptographic_parameters.hashing_algorithm)
        self.assertEqual(None, cryptographic_parameters.key_role_type)
        self.assertEqual(
            None,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            None,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(None, cryptographic_parameters.random_iv)
        self.assertEqual(None, cryptographic_parameters.iv_length)
        self.assertEqual(None, cryptographic_parameters.tag_length)
        self.assertEqual(None, cryptographic_parameters.fixed_field_length)
        self.assertEqual(
            None,
            cryptographic_parameters.invocation_field_length
        )
        self.assertEqual(None, cryptographic_parameters.counter_length)
        self.assertEqual(None, cryptographic_parameters.initial_counter_value)

        cryptographic_parameters.read(self.partial_encoding)

        self.assertEqual(
            enums.BlockCipherMode.CBC,
            cryptographic_parameters.block_cipher_mode
        )
        self.assertEqual(
            enums.PaddingMethod.PKCS5,
            cryptographic_parameters.padding_method
        )
        self.assertEqual(
            enums.HashingAlgorithm.SHA_1,
            cryptographic_parameters.hashing_algorithm
        )
        self.assertEqual(
            enums.KeyRoleType.KEK,
            cryptographic_parameters.key_role_type
        )
        self.assertEqual(
            enums.DigitalSignatureAlgorithm.SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            enums.CryptographicAlgorithm.AES,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(None, cryptographic_parameters.random_iv)
        self.assertEqual(None, cryptographic_parameters.iv_length)
        self.assertEqual(128, cryptographic_parameters.tag_length)
        self.assertEqual(None, cryptographic_parameters.fixed_field_length)
        self.assertEqual(
            None,
            cryptographic_parameters.invocation_field_length
        )
        self.assertEqual(None, cryptographic_parameters.counter_length)
        self.assertEqual(1, cryptographic_parameters.initial_counter_value)

    def test_read_empty(self):
        """
        Test that a CryptographicParameters struct can be read from an empty
        data stream.
        """
        cryptographic_parameters = CryptographicParameters()

        self.assertEqual(None, cryptographic_parameters.block_cipher_mode)
        self.assertEqual(None, cryptographic_parameters.padding_method)
        self.assertEqual(None, cryptographic_parameters.hashing_algorithm)
        self.assertEqual(None, cryptographic_parameters.key_role_type)
        self.assertEqual(
            None,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            None,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(None, cryptographic_parameters.random_iv)
        self.assertEqual(None, cryptographic_parameters.iv_length)
        self.assertEqual(None, cryptographic_parameters.tag_length)
        self.assertEqual(None, cryptographic_parameters.fixed_field_length)
        self.assertEqual(
            None,
            cryptographic_parameters.invocation_field_length
        )
        self.assertEqual(None, cryptographic_parameters.counter_length)
        self.assertEqual(None, cryptographic_parameters.initial_counter_value)

        cryptographic_parameters.read(self.empty_encoding)

        self.assertEqual(None, cryptographic_parameters.block_cipher_mode)
        self.assertEqual(None, cryptographic_parameters.padding_method)
        self.assertEqual(None, cryptographic_parameters.hashing_algorithm)
        self.assertEqual(None, cryptographic_parameters.key_role_type)
        self.assertEqual(
            None,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            None,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(None, cryptographic_parameters.random_iv)
        self.assertEqual(None, cryptographic_parameters.iv_length)
        self.assertEqual(None, cryptographic_parameters.tag_length)
        self.assertEqual(None, cryptographic_parameters.fixed_field_length)
        self.assertEqual(
            None,
            cryptographic_parameters.invocation_field_length
        )
        self.assertEqual(None, cryptographic_parameters.counter_length)
        self.assertEqual(None, cryptographic_parameters.initial_counter_value)

    def test_write(self):
        """
        Test that a CryptographicParameters struct can be written to a data
        stream.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )
        stream = BytearrayStream()
        cryptographic_parameters.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partially defined CryptographicParameters struct can be
        written to a data stream.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            tag_length=128,
            initial_counter_value=1
        )
        stream = BytearrayStream()
        cryptographic_parameters.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty CryptographicParameters struct can be written to a
        data stream.
        """
        cryptographic_parameters = CryptographicParameters()
        stream = BytearrayStream()
        cryptographic_parameters.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        CryptographicParameters structs with the same data.
        """
        a = CryptographicParameters()
        b = CryptographicParameters()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )
        b = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_block_cipher_mode(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different block cipher modes.
        """
        a = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC
        )
        b = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.GCM
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_padding_method(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different padding methods.
        """
        a = CryptographicParameters(padding_method=enums.PaddingMethod.NONE)
        b = CryptographicParameters(padding_method=enums.PaddingMethod.PKCS5)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_hashing_algorithm(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different hashing algorithms.
        """
        a = CryptographicParameters(
            hashing_algorithm=enums.HashingAlgorithm.MD5
        )
        b = CryptographicParameters(
            hashing_algorithm=enums.HashingAlgorithm.SHA_256
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_key_role_type(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different key role types.
        """
        a = CryptographicParameters(key_role_type=enums.KeyRoleType.BDK)
        b = CryptographicParameters(key_role_type=enums.KeyRoleType.KEK)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_digital_signature_algorithm(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different digital signature
        algorithms.
        """
        a = CryptographicParameters(
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            DSA_WITH_SHA1
        )
        b = CryptographicParameters(
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            ECDSA_WITH_SHA1
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_cryptographic_algorithm(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different cryptographic
        algorithms.
        """
        a = CryptographicParameters(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES
        )
        b = CryptographicParameters(
            cryptographic_algorithm=enums.CryptographicAlgorithm.DES
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_random_iv(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different random IVs.
        """
        a = CryptographicParameters(random_iv=True)
        b = CryptographicParameters(random_iv=False)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_iv_length(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different IV lengths.
        """
        a = CryptographicParameters(iv_length=96)
        b = CryptographicParameters(iv_length=128)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_tag_length(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different tag lengths.
        """
        a = CryptographicParameters(tag_length=128)
        b = CryptographicParameters(tag_length=256)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_fixed_field_length(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different fixed field lengths.
        """
        a = CryptographicParameters(fixed_field_length=32)
        b = CryptographicParameters(fixed_field_length=40)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_invocation_field_length(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different invocation field
        lengths.
        """
        a = CryptographicParameters(invocation_field_length=64)
        b = CryptographicParameters(invocation_field_length=80)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_counter_length(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different counter lengths.
        """
        a = CryptographicParameters(counter_length=0)
        b = CryptographicParameters(counter_length=32)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_initial_counter_value(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different counter values.
        """
        a = CryptographicParameters(initial_counter_value=0)
        b = CryptographicParameters(initial_counter_value=1)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        CryptographicParameters structs with different types.
        """
        a = CryptographicParameters()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        CryptographicParameters structs with the same data.
        """
        a = CryptographicParameters()
        b = CryptographicParameters()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )
        b = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_block_cipher_mode(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different block cipher modes.
        """
        a = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC
        )
        b = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.GCM
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_padding_method(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different padding methods.
        """
        a = CryptographicParameters(padding_method=enums.PaddingMethod.NONE)
        b = CryptographicParameters(padding_method=enums.PaddingMethod.PKCS5)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_hashing_algorithm(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different hashing algorithms.
        """
        a = CryptographicParameters(
            hashing_algorithm=enums.HashingAlgorithm.MD5
        )
        b = CryptographicParameters(
            hashing_algorithm=enums.HashingAlgorithm.SHA_256
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_key_role_type(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different key role types.
        """
        a = CryptographicParameters(key_role_type=enums.KeyRoleType.BDK)
        b = CryptographicParameters(key_role_type=enums.KeyRoleType.KEK)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_digital_signature_algorithm(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different digital signature
        algorithms.
        """
        a = CryptographicParameters(
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            DSA_WITH_SHA1
        )
        b = CryptographicParameters(
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            ECDSA_WITH_SHA1
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_cryptographic_algorithm(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different cryptographic
        algorithms.
        """
        a = CryptographicParameters(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES
        )
        b = CryptographicParameters(
            cryptographic_algorithm=enums.CryptographicAlgorithm.DES
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_random_iv(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different random IVs.
        """
        a = CryptographicParameters(random_iv=True)
        b = CryptographicParameters(random_iv=False)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_iv_length(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different IV lengths.
        """
        a = CryptographicParameters(iv_length=96)
        b = CryptographicParameters(iv_length=128)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_tag_length(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different tag lengths.
        """
        a = CryptographicParameters(tag_length=128)
        b = CryptographicParameters(tag_length=256)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_fixed_field_length(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different fixed field lengths.
        """
        a = CryptographicParameters(fixed_field_length=32)
        b = CryptographicParameters(fixed_field_length=40)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_invocation_field_length(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different invocation field
        lengths.
        """
        a = CryptographicParameters(invocation_field_length=64)
        b = CryptographicParameters(invocation_field_length=80)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_counter_length(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different counter lengths.
        """
        a = CryptographicParameters(counter_length=0)
        b = CryptographicParameters(counter_length=32)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_initial_counter_value(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different counter values.
        """
        a = CryptographicParameters(initial_counter_value=0)
        b = CryptographicParameters(initial_counter_value=1)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        CryptographicParameters structs with different types.
        """
        a = CryptographicParameters()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a CryptographicParameters struct.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )

        expected = (
            "CryptographicParameters("
            "block_cipher_mode=BlockCipherMode.CBC, "
            "padding_method=PaddingMethod.PKCS5, "
            "hashing_algorithm=HashingAlgorithm.SHA_1, "
            "key_role_type=KeyRoleType.KEK, "
            "digital_signature_algorithm="
            "DigitalSignatureAlgorithm.SHA256_WITH_RSA_ENCRYPTION, "
            "cryptographic_algorithm=CryptographicAlgorithm.AES, "
            "random_iv=True, "
            "iv_length=96, "
            "tag_length=128, "
            "fixed_field_length=32, "
            "invocation_field_length=64, "
            "counter_length=0, "
            "initial_counter_value=1)"
        )
        observed = repr(cryptographic_parameters)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a GetAttributeList response payload.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )

        expected = str({
            'block_cipher_mode': enums.BlockCipherMode.CBC,
            'padding_method': enums.PaddingMethod.PKCS5,
            'hashing_algorithm': enums.HashingAlgorithm.SHA_1,
            'key_role_type': enums.KeyRoleType.KEK,
            'digital_signature_algorithm':
                enums.DigitalSignatureAlgorithm.SHA256_WITH_RSA_ENCRYPTION,
            'cryptographic_algorithm': enums.CryptographicAlgorithm.AES,
            'random_iv': True,
            'iv_length': 96,
            'tag_length': 128,
            'fixed_field_length': 32,
            'invocation_field_length': 64,
            'counter_length': 0,
            'initial_counter_value': 1
        })
        observed = str(cryptographic_parameters)

        self.assertEqual(expected, observed)


class TestDerivationParameters(TestCase):
    """
    Test suite for the DerivationParameters struct.
    """

    def setUp(self):
        super(TestDerivationParameters, self).setUp()

        # Encoding obtained in part from the KMIP 1.1 testing document,
        # Section 11.1. The rest of the encoding for KMIP 1.2+ features was
        # built by hand; later KMIP testing documents do not include the
        # encoding, so a manual construction is necessary.
        #
        # This encoding matches the following set of values:
        # Cryptographic Parameters
        #     Block Cipher Mode - CBC
        #     Padding Method - PKCS5
        #     Hashing Algorithm - SHA-1
        #     Key Role Type - KEK
        #     Digital Signature Algorithm - SHA-256 with RSA
        #     Cryptographic Algorithm - AES
        #     Random IV - True
        #     IV Length - 96
        #     Tag Length - 128
        #     Fixed Field Length - 32
        #     Invocation Field Length - 64
        #     Counter Length - 0
        #     Initial Counter Value - 1
        # Initialization Vector - 0x39487432492834A3
        # Derivation Data - 0xFAD98B6ACA6D87DD
        # Salt - 0x8F99212AA15435CD
        # Iteration Count - 10000

        self.full_encoding = BytearrayStream(
            b'\x42\x00\x32\x01\x00\x00\x01\x18'
            b'\x42\x00\x2B\x01\x00\x00\x00\xD0'
            b'\x42\x00\x11\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x5F\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\x38\x05\x00\x00\x00\x04\x00\x00\x00\x04\x00\x00\x00\x00'
            b'\x42\x00\x83\x05\x00\x00\x00\x04\x00\x00\x00\x0B\x00\x00\x00\x00'
            b'\x42\x00\xAE\x05\x00\x00\x00\x04\x00\x00\x00\x05\x00\x00\x00\x00'
            b'\x42\x00\x28\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\xC5\x06\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00\x01'
            b'\x42\x00\xCD\x02\x00\x00\x00\x04\x00\x00\x00\x60\x00\x00\x00\x00'
            b'\x42\x00\xCE\x02\x00\x00\x00\x04\x00\x00\x00\x80\x00\x00\x00\x00'
            b'\x42\x00\xCF\x02\x00\x00\x00\x04\x00\x00\x00\x20\x00\x00\x00\x00'
            b'\x42\x00\xD2\x02\x00\x00\x00\x04\x00\x00\x00\x40\x00\x00\x00\x00'
            b'\x42\x00\xD0\x02\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x42\x00\xD1\x02\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
            b'\x42\x00\x3A\x08\x00\x00\x00\x08\x39\x48\x74\x32\x49\x28\x34\xA3'
            b'\x42\x00\x30\x08\x00\x00\x00\x08\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD'
            b'\x42\x00\x84\x08\x00\x00\x00\x08\x8F\x99\x21\x2A\xA1\x54\x35\xCD'
            b'\x42\x00\x3C\x02\x00\x00\x00\x04\x00\x00\x27\x10\x00\x00\x00\x00'
        )

        # Adapted from the full encoding above. This encoding matches the
        # following set of values:
        # Initialization Vector - 0x39487432492834A3
        # Derivation Data - 0xFAD98B6ACA6D87DD

        self.partial_encoding = BytearrayStream(
            b'\x42\x00\x32\x01\x00\x00\x00\x20'
            b'\x42\x00\x3A\x08\x00\x00\x00\x08\x39\x48\x74\x32\x49\x28\x34\xA3'
            b'\x42\x00\x30\x08\x00\x00\x00\x08\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD'
        )

        self.empty_encoding = BytearrayStream(
            b'\x42\x00\x32\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestDerivationParameters, self).tearDown()

    def test_init(self):
        """
        Test that a DerivationParameters struct can be constructed with
        no arguments.
        """
        derivation_parameters = DerivationParameters()

        self.assertEqual(None, derivation_parameters.cryptographic_parameters)
        self.assertEqual(None, derivation_parameters.initialization_vector)
        self.assertEqual(None, derivation_parameters.derivation_data)
        self.assertEqual(None, derivation_parameters.salt)
        self.assertEqual(None, derivation_parameters.iteration_count)

    def test_init_with_args(self):
        """
        Test that a DerivationParameters struct can be constructed with
        valid values.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CTR,
            padding_method=enums.PaddingMethod.NONE,
            hashing_algorithm=enums.HashingAlgorithm.SHA_256,
            key_role_type=enums.KeyRoleType.BDK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA1_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.TRIPLE_DES,
            random_iv=False,
            iv_length=128,
            tag_length=256,
            fixed_field_length=48,
            invocation_field_length=60,
            counter_length=20,
            initial_counter_value=2
        )
        derivation_parameters = DerivationParameters(
            cryptographic_parameters=cryptographic_parameters,
            initialization_vector=b'\x39\x48\x74\x32\x49\x28\x34\xA3',
            derivation_data=b'\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD',
            salt=b'\x8F\x99\x21\x2A\xA1\x54\x35\xCD',
            iteration_count=10000
        )

        self.assertIsInstance(
            derivation_parameters.cryptographic_parameters,
            CryptographicParameters
        )
        parameters = derivation_parameters.cryptographic_parameters
        self.assertEqual(
            enums.BlockCipherMode.CTR,
            parameters.block_cipher_mode
        )
        self.assertEqual(
            enums.PaddingMethod.NONE,
            parameters.padding_method
        )
        self.assertEqual(
            enums.HashingAlgorithm.SHA_256,
            parameters.hashing_algorithm
        )
        self.assertEqual(
            enums.KeyRoleType.BDK,
            parameters.key_role_type
        )
        self.assertEqual(
            enums.DigitalSignatureAlgorithm.SHA1_WITH_RSA_ENCRYPTION,
            parameters.digital_signature_algorithm
        )
        self.assertEqual(
            enums.CryptographicAlgorithm.TRIPLE_DES,
            parameters.cryptographic_algorithm
        )
        self.assertEqual(False, parameters.random_iv)
        self.assertEqual(128, parameters.iv_length)
        self.assertEqual(256, parameters.tag_length)
        self.assertEqual(48, parameters.fixed_field_length)
        self.assertEqual(60, parameters.invocation_field_length)
        self.assertEqual(20, parameters.counter_length)
        self.assertEqual(2, parameters.initial_counter_value)

        self.assertEqual(
            (
                b'\x39\x48\x74\x32\x49\x28\x34\xA3'
            ),
            derivation_parameters.initialization_vector
        )
        self.assertEqual(
            (
                b'\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD'
            ),
            derivation_parameters.derivation_data
        )
        self.assertEqual(
            (
                b'\x8F\x99\x21\x2A\xA1\x54\x35\xCD'
            ),
            derivation_parameters.salt
        )
        self.assertEqual(10000, derivation_parameters.iteration_count)

    def test_invalid_cryptographic_parameters(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the cryptographic parameters of a DerivationParameters struct.
        """
        kwargs = {'cryptographic_parameters': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "cryptographic parameters must be a CryptographicParameters "
            "struct",
            DerivationParameters,
            **kwargs
        )

        derivation_parameters = DerivationParameters()
        args = (derivation_parameters, 'cryptographic_parameters', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "cryptographic parameters must be a CryptographicParameters "
            "struct",
            setattr,
            *args
        )

    def test_invalid_initialization_vector(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the initialization vector of a DerivationParameters struct.
        """
        derivation_parameters = DerivationParameters()
        args = (derivation_parameters, 'initialization_vector', 0)
        self.assertRaisesRegex(
            TypeError,
            "initialization vector must be bytes",
            setattr,
            *args
        )

    def test_invalid_derivation_data(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the derivation data of a DerivationParameters struct.
        """
        derivation_parameters = DerivationParameters()
        args = (derivation_parameters, 'derivation_data', 0)
        self.assertRaisesRegex(
            TypeError,
            "derivation data must be bytes",
            setattr,
            *args
        )

    def test_invalid_salt(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the salt of a DerivationParameters struct.
        """
        derivation_parameters = DerivationParameters()
        args = (derivation_parameters, 'salt', 0)
        self.assertRaisesRegex(
            TypeError,
            "salt must be bytes",
            setattr,
            *args
        )

    def test_invalid_iteration_count(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the iteration count of a DerivationParameters struct.
        """
        derivation_parameters = DerivationParameters()
        args = (derivation_parameters, 'iteration_count', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "iteration count must be an integer",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a DerivationParameters struct can be read from a data
        stream.
        """
        derivation_parameters = DerivationParameters()

        self.assertEqual(None, derivation_parameters.cryptographic_parameters)
        self.assertEqual(None, derivation_parameters.initialization_vector)
        self.assertEqual(None, derivation_parameters.derivation_data)
        self.assertEqual(None, derivation_parameters.salt)
        self.assertEqual(None, derivation_parameters.iteration_count)

        derivation_parameters.read(self.full_encoding)

        self.assertIsInstance(
            derivation_parameters.cryptographic_parameters,
            CryptographicParameters
        )
        cryptographic_parameters = \
            derivation_parameters.cryptographic_parameters
        self.assertEqual(
            enums.BlockCipherMode.CBC,
            cryptographic_parameters.block_cipher_mode
        )
        self.assertEqual(
            enums.PaddingMethod.PKCS5,
            cryptographic_parameters.padding_method
        )
        self.assertEqual(
            enums.HashingAlgorithm.SHA_1,
            cryptographic_parameters.hashing_algorithm
        )
        self.assertEqual(
            enums.KeyRoleType.KEK,
            cryptographic_parameters.key_role_type
        )
        self.assertEqual(
            enums.DigitalSignatureAlgorithm.SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            enums.CryptographicAlgorithm.AES,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(True, cryptographic_parameters.random_iv)
        self.assertEqual(96, cryptographic_parameters.iv_length)
        self.assertEqual(128, cryptographic_parameters.tag_length)
        self.assertEqual(32, cryptographic_parameters.fixed_field_length)
        self.assertEqual(64, cryptographic_parameters.invocation_field_length)
        self.assertEqual(0, cryptographic_parameters.counter_length)
        self.assertEqual(1, cryptographic_parameters.initial_counter_value)

        self.assertEqual(
            (
                b'\x39\x48\x74\x32\x49\x28\x34\xA3'
            ),
            derivation_parameters.initialization_vector
        )
        self.assertEqual(
            (
                b'\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD'
            ),
            derivation_parameters.derivation_data
        )
        self.assertEqual(
            (
                b'\x8F\x99\x21\x2A\xA1\x54\x35\xCD'
            ),
            derivation_parameters.salt
        )
        self.assertEqual(10000, derivation_parameters.iteration_count)

    def test_read_partial(self):
        """
        Test that a DerivationParameters struct can be read from a partial
        data stream.
        """
        derivation_parameters = DerivationParameters()

        self.assertEqual(None, derivation_parameters.cryptographic_parameters)
        self.assertEqual(None, derivation_parameters.initialization_vector)
        self.assertEqual(None, derivation_parameters.derivation_data)
        self.assertEqual(None, derivation_parameters.salt)
        self.assertEqual(None, derivation_parameters.iteration_count)

        derivation_parameters.read(self.partial_encoding)

        self.assertEqual(None, derivation_parameters.cryptographic_parameters)
        self.assertEqual(
            (
                b'\x39\x48\x74\x32\x49\x28\x34\xA3'
            ),
            derivation_parameters.initialization_vector
        )
        self.assertEqual(
            (
                b'\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD'
            ),
            derivation_parameters.derivation_data
        )
        self.assertEqual(None, derivation_parameters.salt)
        self.assertEqual(None, derivation_parameters.iteration_count)

    def test_read_empty(self):
        """
        Test that a DerivationParameters struct can be read from an empty
        data stream.
        """
        derivation_parameters = DerivationParameters()

        self.assertEqual(None, derivation_parameters.cryptographic_parameters)
        self.assertEqual(None, derivation_parameters.initialization_vector)
        self.assertEqual(None, derivation_parameters.derivation_data)
        self.assertEqual(None, derivation_parameters.salt)
        self.assertEqual(None, derivation_parameters.iteration_count)

        derivation_parameters.read(self.empty_encoding)

        self.assertEqual(None, derivation_parameters.cryptographic_parameters)
        self.assertEqual(None, derivation_parameters.initialization_vector)
        self.assertEqual(None, derivation_parameters.derivation_data)
        self.assertEqual(None, derivation_parameters.salt)
        self.assertEqual(None, derivation_parameters.iteration_count)

    def test_write(self):
        """
        Test that a DerivationParameters struct can be written to a data
        stream.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )
        derivation_parameters = DerivationParameters(
            cryptographic_parameters=cryptographic_parameters,
            initialization_vector=b'\x39\x48\x74\x32\x49\x28\x34\xA3',
            derivation_data=b'\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD',
            salt=b'\x8F\x99\x21\x2A\xA1\x54\x35\xCD',
            iteration_count=10000
        )
        stream = BytearrayStream()
        derivation_parameters.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partially defined DerivationParameters struct can be
        written to a data stream.
        """
        derivation_parameters = DerivationParameters(
            initialization_vector=b'\x39\x48\x74\x32\x49\x28\x34\xA3',
            derivation_data=b'\xFA\xD9\x8B\x6A\xCA\x6D\x87\xDD',
        )
        stream = BytearrayStream()
        derivation_parameters.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty DerivationParameters struct can be written to a
        data stream.
        """
        derivation_parameters = DerivationParameters()
        stream = BytearrayStream()
        derivation_parameters.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        DerivationParameters structs with the same data.
        """
        a = DerivationParameters()
        b = DerivationParameters()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.CBC,
                padding_method=enums.PaddingMethod.PKCS5,
                hashing_algorithm=enums.HashingAlgorithm.SHA_1,
                key_role_type=enums.KeyRoleType.KEK,
                digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
                SHA256_WITH_RSA_ENCRYPTION,
                cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
                random_iv=True,
                iv_length=96,
                tag_length=128,
                fixed_field_length=32,
                invocation_field_length=64,
                counter_length=0,
                initial_counter_value=1
            ),
            initialization_vector=b'\x01\x02\x03\x04\x05\x06\x07\x08',
            derivation_data=b'\x11\x22\x33\x44\x55\x66\x77\x88',
            salt=b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0',
            iteration_count=1000
        )
        b = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.CBC,
                padding_method=enums.PaddingMethod.PKCS5,
                hashing_algorithm=enums.HashingAlgorithm.SHA_1,
                key_role_type=enums.KeyRoleType.KEK,
                digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
                SHA256_WITH_RSA_ENCRYPTION,
                cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
                random_iv=True,
                iv_length=96,
                tag_length=128,
                fixed_field_length=32,
                invocation_field_length=64,
                counter_length=0,
                initial_counter_value=1
            ),
            initialization_vector=b'\x01\x02\x03\x04\x05\x06\x07\x08',
            derivation_data=b'\x11\x22\x33\x44\x55\x66\x77\x88',
            salt=b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0',
            iteration_count=1000
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_cryptographic_parameters(self):
        """
        Test that the equality operator returns False when comparing two
        DerivationParameters structs with different cryptographic parameters.
        """
        a = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.CBC
            )
        )
        b = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.GCM
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_initialization_vector(self):
        """
        Test that the equality operator returns False when comparing two
        DerivationParameters structs with different initialization vectors.
        """
        a = DerivationParameters(initialization_vector=b'\x01')
        b = DerivationParameters(initialization_vector=b'\x02')

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_derivation_data(self):
        """
        Test that the equality operator returns False when comparing two
        DerivationParameters structs with different derivation data.
        """
        a = DerivationParameters(derivation_data=b'\x01')
        b = DerivationParameters(derivation_data=b'\x02')

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_salt(self):
        """
        Test that the equality operator returns False when comparing two
        DerivationParameters structs with different salts.
        """
        a = DerivationParameters(salt=b'\x01')
        b = DerivationParameters(salt=b'\x02')

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_iteration_count(self):
        """
        Test that the equality operator returns False when comparing two
        DerivationParameters structs with different iteration counts.
        """
        a = DerivationParameters(iteration_count=1)
        b = DerivationParameters(iteration_count=2)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        DerivationParameters structs with different types.
        """
        a = DerivationParameters()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        DerivationParameters structs with the same data.
        """
        a = DerivationParameters()
        b = DerivationParameters()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.CBC,
                padding_method=enums.PaddingMethod.PKCS5,
                hashing_algorithm=enums.HashingAlgorithm.SHA_1,
                key_role_type=enums.KeyRoleType.KEK,
                digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
                SHA256_WITH_RSA_ENCRYPTION,
                cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
                random_iv=True,
                iv_length=96,
                tag_length=128,
                fixed_field_length=32,
                invocation_field_length=64,
                counter_length=0,
                initial_counter_value=1
            ),
            initialization_vector=b'\x01\x02\x03\x04\x05\x06\x07\x08',
            derivation_data=b'\x11\x22\x33\x44\x55\x66\x77\x88',
            salt=b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0',
            iteration_count=1000
        )
        b = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.CBC,
                padding_method=enums.PaddingMethod.PKCS5,
                hashing_algorithm=enums.HashingAlgorithm.SHA_1,
                key_role_type=enums.KeyRoleType.KEK,
                digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
                SHA256_WITH_RSA_ENCRYPTION,
                cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
                random_iv=True,
                iv_length=96,
                tag_length=128,
                fixed_field_length=32,
                invocation_field_length=64,
                counter_length=0,
                initial_counter_value=1
            ),
            initialization_vector=b'\x01\x02\x03\x04\x05\x06\x07\x08',
            derivation_data=b'\x11\x22\x33\x44\x55\x66\x77\x88',
            salt=b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0',
            iteration_count=1000
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_cryptographic_parameters(self):
        """
        Test that the inequality operator returns True when comparing two
        DerivationParameters structs with different cryptographic parameters.
        """
        a = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.CBC
            )
        )
        b = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.GCM
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_initialization_vectors(self):
        """
        Test that the inequality operator returns True when comparing two
        DerivationParameters structs with different initialization vectors.
        """
        a = DerivationParameters(initialization_vector=b'\x01')
        b = DerivationParameters(initialization_vector=b'\x02')

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_derivation_data(self):
        """
        Test that the inequality operator returns True when comparing two
        DerivationParameters structs with different derivation data.
        """
        a = DerivationParameters(derivation_data=b'\x01')
        b = DerivationParameters(derivation_data=b'\x02')

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_salt(self):
        """
        Test that the inequality operator returns True when comparing two
        DerivationParameters structs with different salts.
        """
        a = DerivationParameters(salt=b'\x01')
        b = DerivationParameters(salt=b'\x02')

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_iteration_counts(self):
        """
        Test that the inequality operator returns True when comparing two
        DerivationParameters structs with different iteration counts.
        """
        a = DerivationParameters(iteration_count=1)
        b = DerivationParameters(iteration_count=2)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        DerivationParameters structs with different types.
        """
        a = DerivationParameters()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a DerivationParameters struct.
        """
        derivation_parameters = DerivationParameters(
            cryptographic_parameters=CryptographicParameters(
                block_cipher_mode=enums.BlockCipherMode.CBC,
                padding_method=enums.PaddingMethod.PKCS5,
                hashing_algorithm=enums.HashingAlgorithm.SHA_1,
                key_role_type=enums.KeyRoleType.KEK,
                digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
                SHA256_WITH_RSA_ENCRYPTION,
                cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
                random_iv=True,
                iv_length=96,
                tag_length=128,
                fixed_field_length=32,
                invocation_field_length=64,
                counter_length=0,
                initial_counter_value=1
            ),
            initialization_vector=b'\x01\x02\x03\x04\x05\x06\x07\x08',
            derivation_data=b'\x11\x22\x33\x44\x55\x66\x77\x88',
            salt=b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0',
            iteration_count=10000
        )

        expected = (
            "DerivationParameters("
            "cryptographic_parameters=CryptographicParameters("
            "block_cipher_mode=BlockCipherMode.CBC, "
            "padding_method=PaddingMethod.PKCS5, "
            "hashing_algorithm=HashingAlgorithm.SHA_1, "
            "key_role_type=KeyRoleType.KEK, "
            "digital_signature_algorithm="
            "DigitalSignatureAlgorithm.SHA256_WITH_RSA_ENCRYPTION, "
            "cryptographic_algorithm=CryptographicAlgorithm.AES, "
            "random_iv=True, "
            "iv_length=96, "
            "tag_length=128, "
            "fixed_field_length=32, "
            "invocation_field_length=64, "
            "counter_length=0, "
            "initial_counter_value=1), "
            "initialization_vector=" + str(
                b'\x01\x02\x03\x04\x05\x06\x07\x08'
            ) + ", "
            "derivation_data=" + str(
                b'\x11\x22\x33\x44\x55\x66\x77\x88'
            ) + ", "
            "salt=" + str(b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0') + ", "
            "iteration_count=10000)"
        )
        observed = repr(derivation_parameters)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a DerivationParameters struct.
        """
        cryptographic_parameters = CryptographicParameters(
            block_cipher_mode=enums.BlockCipherMode.CBC,
            padding_method=enums.PaddingMethod.PKCS5,
            hashing_algorithm=enums.HashingAlgorithm.SHA_1,
            key_role_type=enums.KeyRoleType.KEK,
            digital_signature_algorithm=enums.DigitalSignatureAlgorithm.
            SHA256_WITH_RSA_ENCRYPTION,
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            random_iv=True,
            iv_length=96,
            tag_length=128,
            fixed_field_length=32,
            invocation_field_length=64,
            counter_length=0,
            initial_counter_value=1
        )
        derivation_parameters = DerivationParameters(
            cryptographic_parameters=cryptographic_parameters,
            initialization_vector=b'\x01\x02\x03\x04\x05\x06\x07\x08',
            derivation_data=b'\x11\x22\x33\x44\x55\x66\x77\x88',
            salt=b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0',
            iteration_count=10000
        )

        expected = str({
            'cryptographic_parameters': cryptographic_parameters,
            'initialization_vector': b'\x01\x02\x03\x04\x05\x06\x07\x08',
            'derivation_data': b'\x11\x22\x33\x44\x55\x66\x77\x88',
            'salt': b'\x12\x34\x56\x78\x9A\xBC\xDE\xF0',
            'iteration_count': 10000
        })
        observed = str(derivation_parameters)

        self.assertEqual(expected, observed)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import attributes
from kmip.core import enums
from kmip.core.enums import Tags
from kmip.core.messages.payloads import base
from kmip.core.utils import BytearrayStream


# 4.21
class DestroyRequestPayload(base.RequestPayload):

    def __init__(self,
                 unique_identifier=None):
        super(DestroyRequestPayload, self).__init__()
        self.unique_identifier = unique_identifier
        self.validate()

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        super(DestroyRequestPayload, self).read(
            istream,
            kmip_version=kmip_version
        )
        tstream = BytearrayStream(istream.read(self.length))

        if self.is_tag_next(Tags.UNIQUE_IDENTIFIER, tstream):
            self.unique_identifier = attributes.UniqueIdentifier()
            self.unique_identifier.read(tstream, kmip_version=kmip_version)

        self.is_oversized(tstream)
        self.validate()

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        tstream = BytearrayStream()

        if self.unique_identifier is not None:
            self.unique_identifier.write(tstream, kmip_version=kmip_version)

        # Write the length and value of the request payload
        self.length = tstream.length()
        super(DestroyRequestPayload, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)

    def validate(self):
        self.__validate()

    def __validate(self):
        # TODO (peter-hamilton) Finish implementation.
        pass


class DestroyResponsePayload(base.ResponsePayload):

    def __init__(self,
                 unique_identifier=None):
        super(DestroyResponsePayload, self).__init__()
        self.unique_identifier = unique_identifier
        self.validate()

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        super(DestroyResponsePayload, self).read(
            istream,
            kmip_version=kmip_version
        )
        tstream = BytearrayStream(istream.read(self.length))

        self.unique_identifier = attributes.UniqueIdentifier()
        self.unique_identifier.read(tstream, kmip_version=kmip_version)

        self.is_oversized(tstream)
        self.validate()

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        tstream = BytearrayStream()

        self.unique_identifier.write(tstream, kmip_version=kmip_version)

        # Write the length and value of the request payload
        self.length = tstream.length()
        super(DestroyResponsePayload, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)

    def validate(self):
        self.__validate()

    def __validate(self):
        # TODO (peter-hamilton) Finish implementation.
        pass



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import TestCase

from kmip.core import primitives
from kmip.core import utils


class TestBoolean(TestCase):

    def setUp(self):
        super(TestBoolean, self).setUp()
        self.stream = utils.BytearrayStream()

    def tearDown(self):
        super(TestBoolean, self).tearDown()

    def test_init(self):
        """
        Test that a Boolean object can be instantiated.
        """
        boolean = primitives.Boolean(False)
        self.assertEqual(False, boolean.value)

    def test_init_unset(self):
        """
        Test that a Boolean object can be instantiated with no input.
        """
        boolean = primitives.Boolean()
        self.assertEqual(True, boolean.value)

    def test_validate_on_valid(self):
        """
        Test that a Boolean object can be validated on good input.
        """
        boolean = primitives.Boolean(True)
        boolean.validate()

    def test_validate_on_valid_unset(self):
        """
        Test that a Boolean object with no preset value can be validated.
        """
        boolean = primitives.Boolean()
        boolean.validate()

    def test_validate_on_invalid_type(self):
        """
        Test that a TypeError is raised when a Boolean object is built with an
        invalid value.
        """
        self.assertRaises(TypeError, primitives.Boolean, 'invalid')

    def test_read_true(self):
        """
        Test that a Boolean object representing the value True can be read
        from a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x06\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x01')
        stream = utils.BytearrayStream(encoding)
        boolean = primitives.Boolean()
        boolean.read(stream)
        self.assertTrue(boolean.value)

    def test_read_false(self):
        """
        Test that a Boolean object representing the value False can be read
        from a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x06\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x00')
        stream = utils.BytearrayStream(encoding)
        boolean = primitives.Boolean()
        boolean.read(stream)
        self.assertFalse(boolean.value)

    def test_read_bad_encoding(self):
        """
        Test that an Exception is raised when the Boolean read operation fails
        on a bad encoding.
        """
        encoding = (b'\x42\x00\x00\x06\x00\x00\x00\x08')
        stream = utils.BytearrayStream(encoding)
        boolean = primitives.Boolean()
        self.assertRaises(Exception, boolean.read, stream)

    def test_read_bad_value(self):
        """
        Test that a ValueError is raised when the Boolean read operations
        reads a valid integer but invalid Boolean.
        """
        encoding = (
            b'\x42\x00\x00\x06\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x02')
        stream = utils.BytearrayStream(encoding)
        boolean = primitives.Boolean()
        self.assertRaises(ValueError, boolean.read, stream)

    def test_write_true(self):
        """
        Test that a Boolean object representing the value True can be written
        to a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x06\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x01')
        stream = utils.BytearrayStream()
        boolean = primitives.Boolean(True)
        boolean.write(stream)
        self.assertEqual(encoding, stream.read())

    def test_write_false(self):
        """
        Test that a Boolean object representing the value False can be written
        to a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x06\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x00')
        stream = utils.BytearrayStream()
        boolean = primitives.Boolean(False)
        boolean.write(stream)
        self.assertEqual(encoding, stream.read())

    def test_write_bad_value(self):
        """
        Test that an Exception is raised when the Boolean write operation fails
        on a bad Boolean value.
        """
        stream = utils.BytearrayStream()
        boolean = primitives.Boolean()
        boolean.value = 'invalid'
        self.assertRaises(Exception, boolean.write, stream)

    def test_repr_default(self):
        """
        Test that the representation of a Boolean object is formatted properly
        """
        boolean = primitives.Boolean()
        self.assertEqual("Boolean(value=True)", repr(boolean))

    def test_repr_true(self):
        """
        Test that the representation of a Boolean object representing the
        value True is formatted properly.
        """
        boolean = primitives.Boolean(True)
        self.assertEqual("Boolean(value=True)", repr(boolean))

    def test_repr_false(self):
        """
        Test that the representation of a Boolean object representing the
        value False is formatted properly.
        """
        boolean = primitives.Boolean(False)
        self.assertEqual("Boolean(value=False)", repr(boolean))

    def test_str_default(self):
        """
        Test that the string representation of a Boolean object is formatted
        properly.
        """
        self.assertEqual("True", str(primitives.Boolean()))

    def test_str_true(self):
        """
        Test that the string representation of a Boolean object representing
        the value True is formatted properly.
        """
        self.assertEqual("True", str(primitives.Boolean(True)))

    def test_str_false(self):
        """
        Test that the string representation of a Boolean object representing
        the value False is formatted properly.
        """
        self.assertEqual("False", str(primitives.Boolean(False)))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        Boolean objects.
        """
        a = primitives.Boolean(False)
        b = primitives.Boolean(False)

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_equal_and_empty(self):
        """
        Test that the equality operator returns True when comparing two
        Boolean objects.
        """
        a = primitives.Boolean()
        b = primitives.Boolean()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal(self):
        """
        Test that the equality operator returns False when comparing two
        Boolean objects with different values.
        """
        a = primitives.Boolean(True)
        b = primitives.Boolean(False)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing a
        Boolean object to a non-Boolean object.
        """
        a = primitives.Boolean()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing
        two Boolean objects with the same values.
        """
        a = primitives.Boolean(False)
        b = primitives.Boolean(False)

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_equal_and_empty(self):
        """
        Test that the inequality operator returns False when comparing
        two Boolean objects.
        """
        a = primitives.Boolean()
        b = primitives.Boolean()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal(self):
        """
        Test that the inequality operator returns True when comparing two
        Boolean objects with different values.
        """
        a = primitives.Boolean(True)
        b = primitives.Boolean(False)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing a
        Boolean object to a non-Boolean object.
        """
        a = primitives.Boolean()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from six import string_types
from testtools import TestCase

from kmip.core.misc import ServerInformation
from kmip.core.utils import BytearrayStream


class TestServerInformation(TestCase):
    """
    A test suite for the ServerInformation class.
    """

    def setUp(self):
        super(TestServerInformation, self).setUp()

        self.data = BytearrayStream(b'\x00\x01\x02\x03')

        self.encoding_a = BytearrayStream(
            b'\x42\x00\x88\x01\x00\x00\x00\x00')
        self.encoding_b = BytearrayStream(
            b'\x42\x00\x88\x01\x00\x00\x00\x04\x00\x01\x02\x03')

    def tearDown(self):
        super(TestServerInformation, self).tearDown()

    def test_init(self):
        ServerInformation()

    def _test_read(self, stream, data):
        server_information = ServerInformation()
        server_information.read(stream)

        expected = data
        observed = server_information.data

        msg = "data decoding mismatch"
        msg += "; expected {0}, observed {1}".format(expected, observed)
        self.assertEqual(expected, observed, msg)

    def test_read_with_none(self):
        """
        Test that a ServerInformation object with no data can be read from a
        data stream.
        """
        self._test_read(self.encoding_a, BytearrayStream())

    def test_read_with_data(self):
        """
        Test that a ServerInformation object with data can be read from a
        data stream.
        """
        self._test_read(self.encoding_b, self.data)

    def _test_write(self, stream_expected, data):
        stream_observed = BytearrayStream()
        server_information = ServerInformation()

        if data is not None:
            server_information.data = data

        server_information.write(stream_observed)

        length_expected = len(stream_expected)
        length_observed = len(stream_observed)

        msg = "encoding lengths not equal"
        msg += "; expected {0}, observed {1}".format(
            length_expected, length_observed)
        self.assertEqual(length_expected, length_observed, msg)

        msg = "encoding mismatch"
        msg += ";\nexpected:\n{0}\nobserved:\n{1}".format(
            stream_expected, stream_observed)
        self.assertEqual(stream_expected, stream_observed, msg)

    def test_write_with_none(self):
        """
        Test that a ServerInformation object with no data can be written to a
        data stream.
        """
        self._test_write(self.encoding_a, None)

    def test_write_with_data(self):
        """
        Test that a ServerInformation object with data can be written to a
        data stream.
        """
        self._test_write(self.encoding_b, self.data)

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        ServerInformation objects with the same internal data.
        """
        a = ServerInformation()
        b = ServerInformation()

        a.data = self.data
        b.data = self.data

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_equal_and_empty(self):
        """
        Test that the equality operator returns True when comparing two
        ServerInformation objects with no internal data.
        """
        a = ServerInformation()
        b = ServerInformation()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal(self):
        """
        Test that the equality operator returns False when comparing two
        ServerInformation objects with different sets of internal data.
        """
        a = ServerInformation()
        b = ServerInformation()

        a.data = self.data

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing a
        ServerInformation object to a non-ServerInformation object.
        """
        a = ServerInformation()
        b = "invalid"

        self.assertFalse(a == b)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing
        two ServerInformation objects with the same internal data.
        """
        a = ServerInformation()
        b = ServerInformation()

        a.data = self.data
        b.data = self.data

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_equal_and_empty(self):
        """
        Test that the inequality operator returns False when comparing
        two ServerInformation objects with no internal data.
        """
        a = ServerInformation()
        b = ServerInformation()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal(self):
        """
        Test that the inequality operator returns True when comparing two
        ServerInformation objects with different sets of internal data.
        """
        a = ServerInformation()
        b = ServerInformation()

        a.data = self.data

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing a
        ServerInformation object to a non-ServerInformation object.
        """
        a = ServerInformation()
        b = "invalid"

        self.assertTrue(a != b)

    def test_repr(self):
        """
        Test that the representation of a ServerInformation object is
        formatted properly and can be used by eval to create a new
        ServerInformation object.
        """
        server_information = ServerInformation()

        expected = "ServerInformation()"
        observed = repr(server_information)

        msg = "expected {0}, observed {1}".format(expected, observed)
        self.assertEqual(expected, observed, msg)

        expected = server_information
        observed = eval(observed)

        msg = "expected {0}, observed {1}".format(expected, observed)
        self.assertEqual(expected, observed, msg)

    def _test_str(self, data):
        server_information = ServerInformation()
        server_information.data = data
        str_repr = str(server_information)

        expected = len(str(data))
        observed = len(str_repr)

        msg = "expected {0}, observed {1}".format(expected, observed)
        self.assertEqual(expected, observed, msg)

        # TODO (peter-hamilton) This should be binary_type. Fix involves
        # TODO (peter-hamilton) refining BytearrayStream implementation.
        expected = string_types
        observed = str_repr

        msg = "expected {0}, observed {1}".format(expected, observed)
        self.assertIsInstance(observed, expected, msg)

    def test_str_with_no_data(self):
        """
        Test that the string representation of a ServerInformation object
        is formatted properly when there is no internal data.
        """
        self._test_str(BytearrayStream())

    def test_str_with_data(self):
        """
        Test that the string representation of a ServerInformation object
        is formatted properly when there is internal data.
        """
        self._test_str(self.data)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from kmip.core import enums
from kmip.core import objects
from kmip.core import utils

from kmip.core import primitives
from kmip.core.primitives import Struct
from kmip.core.primitives import Integer
from kmip.core.primitives import Enumeration
from kmip.core.primitives import Boolean
from kmip.core.primitives import TextString
from kmip.core.primitives import ByteString
from kmip.core.primitives import DateTime


class ProtocolVersion(primitives.Struct):
    """
    A struct representing a ProtocolVersion number.

    Attributes:
        major: The major protocol version number.
        minor: The minor protocol version number.
    """

    def __init__(self, major=None, minor=None):
        """
        Construct a ProtocolVersion struct.

        Args:
            major (int): The major protocol version number. Optional, defaults
                to None.
            minor (int): The minor protocol version number. Optional, defaults
                to None.
        """
        super(ProtocolVersion, self).__init__(enums.Tags.PROTOCOL_VERSION)

        self._major = None
        self._minor = None

        self.major = major
        self.minor = minor

    @property
    def major(self):
        if self._major:
            return self._major.value
        else:
            return None

    @major.setter
    def major(self, value):
        if value is None:
            self._major = None
        elif isinstance(value, six.integer_types):
            self._major = primitives.Integer(
                value=value,
                tag=enums.Tags.PROTOCOL_VERSION_MAJOR
            )
        else:
            raise TypeError(
                "Major protocol version number must be an integer."
            )

    @property
    def minor(self):
        if self._minor:
            return self._minor.value
        else:
            return None

    @minor.setter
    def minor(self, value):
        if value is None:
            self._minor = None
        elif isinstance(value, six.integer_types):
            self._minor = primitives.Integer(
                value=value,
                tag=enums.Tags.PROTOCOL_VERSION_MINOR
            )
        else:
            raise TypeError(
                "Minor protocol version number must be an integer."
            )

    def read(self, input_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the ProtocolVersion struct and decode it into
        its constituent parts.

        Args:
            input_stream (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if either the major or minor protocol versions
                are missing from the encoding.
        """
        super(ProtocolVersion, self).read(
            input_stream,
            kmip_version=kmip_version
        )
        local_stream = utils.BytearrayStream(input_stream.read(self.length))

        if self.is_tag_next(enums.Tags.PROTOCOL_VERSION_MAJOR, local_stream):
            self._major = primitives.Integer(
                tag=enums.Tags.PROTOCOL_VERSION_MAJOR
            )
            self._major.read(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError(
                "Invalid encoding missing the major protocol version number."
            )

        if self.is_tag_next(enums.Tags.PROTOCOL_VERSION_MINOR, local_stream):
            self._minor = primitives.Integer(
                tag=enums.Tags.PROTOCOL_VERSION_MINOR
            )
            self._minor.read(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError(
                "Invalid encoding missing the minor protocol version number."
            )

        self.is_oversized(local_stream)

    def write(self, output_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the ProtocolVersion struct to a stream.

        Args:
            output_stream (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if the data attribute is not defined.
        """
        local_stream = utils.BytearrayStream()

        if self._major:
            self._major.write(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError(
                "Invalid struct missing the major protocol version number."
            )

        if self._minor:
            self._minor.write(local_stream, kmip_version=kmip_version)
        else:
            raise ValueError(
                "Invalid struct missing the minor protocol version number."
            )

        self.length = local_stream.length()
        super(ProtocolVersion, self).write(
            output_stream,
            kmip_version=kmip_version
        )
        output_stream.write(local_stream.buffer)

    def __eq__(self, other):
        if isinstance(other, ProtocolVersion):
            if self.major != other.major:
                return False
            elif self.minor != other.minor:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, ProtocolVersion):
            return not (self == other)
        else:
            return NotImplemented

    def __lt__(self, other):
        if isinstance(other, ProtocolVersion):
            if self.major < other.major:
                return True
            elif self.major > other.major:
                return False
            elif self.minor < other.minor:
                return True
            else:
                return False
        else:
            return NotImplemented

    def __gt__(self, other):
        if isinstance(other, ProtocolVersion):
            if (self == other) or (self < other):
                return False
            else:
                return True
        else:
            return NotImplemented

    def __le__(self, other):
        if isinstance(other, ProtocolVersion):
            if (self == other) or (self < other):
                return True
            else:
                return False
        else:
            return NotImplemented

    def __ge__(self, other):
        if isinstance(other, ProtocolVersion):
            if (self == other) or (self > other):
                return True
            else:
                return False
        else:
            return NotImplemented

    def __repr__(self):
        args = ", ".join([
            "major={}".format(self.major),
            "minor={}".format(self.minor)
        ])
        return "ProtocolVersion({})".format(args)

    def __str__(self):
        return "{}.{}".format(self.major, self.minor)


def protocol_version_to_kmip_version(value):
    """
    Convert a ProtocolVersion struct to its KMIPVersion enumeration equivalent.

    Args:
        value (ProtocolVersion): A ProtocolVersion struct to be converted into
            a KMIPVersion enumeration.

    Returns:
        KMIPVersion: The enumeration equivalent of the struct. If the struct
            cannot be converted to a valid enumeration, None is returned.
    """
    if not isinstance(value, ProtocolVersion):
        return None

    if value.major == 1:
        if value.minor == 0:
            return enums.KMIPVersion.KMIP_1_0
        elif value.minor == 1:
            return enums.KMIPVersion.KMIP_1_1
        elif value.minor == 2:
            return enums.KMIPVersion.KMIP_1_2
        elif value.minor == 3:
            return enums.KMIPVersion.KMIP_1_3
        elif value.minor == 4:
            return enums.KMIPVersion.KMIP_1_4
        else:
            return None
    elif value.major == 2:
        if value.minor == 0:
            return enums.KMIPVersion.KMIP_2_0
        else:
            return None
    else:
        return None


# 6.2
class Operation(Enumeration):

    def __init__(self, value=None):
        super(Operation, self).__init__(
            enums.Operation, value, enums.Tags.OPERATION)


# 6.3
class MaximumResponseSize(Integer):
    def __init__(self, value=None):
        super(MaximumResponseSize, self).\
            __init__(value, enums.Tags.MAXIMUM_RESPONSE_SIZE)


# 6.4
class UniqueBatchItemID(ByteString):
    def __init__(self, value=None):
        super(UniqueBatchItemID, self)\
            .__init__(value, enums.Tags.UNIQUE_BATCH_ITEM_ID)


# 6.5
class TimeStamp(DateTime):
    def __init__(self, value=None):
        super(TimeStamp, self).__init__(value, enums.Tags.TIME_STAMP)


class Authentication(Struct):
    """
    A struct representing an Authentication bundle.

    Attributes:
        credentials: A list of Credential structs to be used for
            authentication.
    """

    def __init__(self, credentials=None):
        """
        Construct an Authentication struct.

        Args:
            credentials (list): A list of Credential structs to be used for
                authentication. Optional, defaults to None.
        """
        super(Authentication, self).__init__(enums.Tags.AUTHENTICATION)

        self._credentials = []
        self.credentials = credentials

    @property
    def credentials(self):
        return self._credentials

    @credentials.setter
    def credentials(self, value):
        if value is None:
            self._credentials = []
        elif isinstance(value, list):
            credentials = []
            for i in range(len(value)):
                credential = value[i]
                if not isinstance(credential, objects.Credential):
                    raise TypeError(
                        "Credentials must be a list of Credential structs. "
                        "Item {} has type: {}".format(i + 1, type(credential))
                    )
                credentials.append(credential)
            self._credentials = credentials
        else:
            raise TypeError(
                "Credentials must be a list of Credential structs."
            )

    def read(self, input_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the Authentication struct and decode it into
        its constituent parts.

        Args:
            input_stream (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.
        """
        super(Authentication, self).read(
            input_stream,
            kmip_version=kmip_version
        )
        local_stream = utils.BytearrayStream(input_stream.read(self.length))

        credentials = []
        while self.is_tag_next(enums.Tags.CREDENTIAL, local_stream):
            credential = objects.Credential()
            credential.read(local_stream, kmip_version=kmip_version)
            credentials.append(credential)
        if len(credentials) == 0:
            raise ValueError("Authentication encoding missing credentials.")
        self._credentials = credentials

        self.is_oversized(local_stream)

    def write(self, output_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the Authentication struct to a stream.

        Args:
            output_stream (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.
        """
        local_stream = utils.BytearrayStream()

        if len(self._credentials) == 0:
            raise ValueError("Authentication struct missing credentials.")
        for credential in self._credentials:
            credential.write(local_stream, kmip_version=kmip_version)

        self.length = local_stream.length()
        super(Authentication, self).write(
            output_stream,
            kmip_version=kmip_version
        )
        output_stream.write(local_stream.buffer)

    def __eq__(self, other):
        if isinstance(other, Authentication):
            if self.credentials != other.credentials:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, Authentication):
            return not (self == other)
        else:
            return NotImplemented

    def __repr__(self):
        args = ", ".join([
            "credentials={}".format([x for x in self.credentials])
        ])
        return "Authentication({})".format(args)

    def __str__(self):
        credentials = ", ".join([str(x) for x in self.credentials])
        return "{'credentials': [" + credentials + "]}"


# 6.7
class AsynchronousIndicator(Boolean):
    def __init__(self, value=None):
        super(AsynchronousIndicator, self).\
            __init__(value, enums.Tags.ASYNCHRONOUS_INDICATOR)


# 6.8
class AsynchronousCorrelationValue(ByteString):
    def __init__(self, value=None):
        super(AsynchronousCorrelationValue, self).\
            __init__(value, enums.Tags.ASYNCHRONOUS_CORRELATION_VALUE)


# 6.9
class ResultStatus(Enumeration):

    def __init__(self, value=None):
        super(ResultStatus, self).__init__(
            enums.ResultStatus, value, enums.Tags.RESULT_STATUS)


# 6.10
class ResultReason(Enumeration):

    def __init__(self, value=None):
        super(ResultReason, self).__init__(
            enums.ResultReason, value, enums.Tags.RESULT_REASON)


# 6.11
class ResultMessage(TextString):
    def __init__(self, value=None):
        super(ResultMessage, self).__init__(value, enums.Tags.RESULT_MESSAGE)


# 6.12
class BatchOrderOption(Boolean):
    def __init__(self, value=None):
        super(BatchOrderOption, self).\
            __init__(value, enums.Tags.BATCH_ORDER_OPTION)


# 6.13
class BatchErrorContinuationOption(Enumeration):

    def __init__(self, value=None):
        super(BatchErrorContinuationOption, self).__init__(
            enums.BatchErrorContinuationOption, value,
            enums.Tags.BATCH_ERROR_CONTINUATION_OPTION)


# 6.14
class BatchCount(Integer):
    def __init__(self, value=None):
        super(BatchCount, self).__init__(value, enums.Tags.BATCH_COUNT)


# 6.16
class MessageExtension(Struct):
    def __init__(self):
        super(MessageExtension, self).__init__(enums.Tags.MESSAGE_EXTENSION)


# 9.1.3.2.2
class KeyCompressionType(Enumeration):

    def __init__(self, value=None):
        super(KeyCompressionType, self).__init__(
            enums.KeyCompressionType, value, enums.Tags.KEY_COMPRESSION_TYPE)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import attributes
from kmip.core import exceptions
from kmip.core import utils


class TestApplicationSpecificInformation(testtools.TestCase):
    """
    A test suite for the ApplicationSpecificInformation class.
    """

    def setUp(self):
        super(TestApplicationSpecificInformation, self).setUp()

        # This encoding was taken from test case 3.1.2 from the KMIP 1.1 test
        # document.
        #
        # This encoding matches the following set of values:
        # Application Specific Information
        #     Application Namespace - ssl
        #     Application Data - www.example.com
        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x04\x01\x00\x00\x00\x28'
            b'\x42\x00\x03\x07\x00\x00\x00\x03\x73\x73\x6C\x00\x00\x00\x00\x00'
            b'\x42\x00\x02\x07\x00\x00\x00\x0F'
            b'\x77\x77\x77\x2E\x65\x78\x61\x6D\x70\x6C\x65\x2E\x63\x6F\x6D\x00'
        )

        # This encoding was adapted from test case 3.1.2 from the KMIP 1.1 test
        # document.
        #
        # This encoding matches the following set of values:
        # Application Specific Information
        #     Application Data - www.example.com
        self.no_application_namespace_encoding = utils.BytearrayStream(
            b'\x42\x00\x04\x01\x00\x00\x00\x18'
            b'\x42\x00\x02\x07\x00\x00\x00\x0F'
            b'\x77\x77\x77\x2E\x65\x78\x61\x6D\x70\x6C\x65\x2E\x63\x6F\x6D\x00'
        )

        # This encoding was adapted from test case 3.1.2 from the KMIP 1.1 test
        # document.
        #
        # This encoding matches the following set of values:
        # Application Specific Information
        #     Application Namespace - ssl
        self.no_application_data_encoding = utils.BytearrayStream(
            b'\x42\x00\x04\x01\x00\x00\x00\x10'
            b'\x42\x00\x03\x07\x00\x00\x00\x03\x73\x73\x6C\x00\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestApplicationSpecificInformation, self).tearDown()

    def test_init(self):
        """
        Test that an ApplicationSpecificInformation object can be constructed.
        """
        app_specific_info = attributes.ApplicationSpecificInformation()

        self.assertIsNone(app_specific_info.application_namespace)
        self.assertIsNone(app_specific_info.application_data)

        app_specific_info = attributes.ApplicationSpecificInformation(
            application_namespace="namespace",
            application_data="data"
        )

        self.assertEqual("namespace", app_specific_info.application_namespace)
        self.assertEqual("data", app_specific_info.application_data)

    def test_invalid_application_namespace(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the application namespace of an ApplicationSpecificInformation object.
        """
        kwargs = {"application_namespace": []}
        self.assertRaisesRegex(
            TypeError,
            "The application namespace must be a string.",
            attributes.ApplicationSpecificInformation,
            **kwargs
        )

        args = (
            attributes.ApplicationSpecificInformation(),
            "application_namespace",
            []
        )
        self.assertRaisesRegex(
            TypeError,
            "The application namespace must be a string.",
            setattr,
            *args
        )

    def test_invalid_application_data(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the application data of an ApplicationSpecificInformation object.
        """
        kwargs = {"application_data": []}
        self.assertRaisesRegex(
            TypeError,
            "The application data must be a string.",
            attributes.ApplicationSpecificInformation,
            **kwargs
        )

        args = (
            attributes.ApplicationSpecificInformation(),
            "application_data",
            []
        )
        self.assertRaisesRegex(
            TypeError,
            "The application data must be a string.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that an ApplicationSpecificInformation object can be read from a
        buffer.
        """
        app_specific_info = attributes.ApplicationSpecificInformation()

        self.assertIsNone(app_specific_info.application_namespace)
        self.assertIsNone(app_specific_info.application_data)

        app_specific_info.read(self.full_encoding)

        self.assertEqual("ssl", app_specific_info.application_namespace)
        self.assertEqual("www.example.com", app_specific_info.application_data)

    def test_read_missing_application_namespace(self):
        """
        Test that an InvalidKmipEncoding error is raised during the decoding of
        an ApplicationSpecificInformation object with the application namespace
        is missing from the encoding.
        """
        app_specific_info = attributes.ApplicationSpecificInformation()

        self.assertIsNone(app_specific_info.application_namespace)

        args = (self.no_application_namespace_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The ApplicationSpecificInformation encoding is missing the "
            "ApplicationNamespace field.",
            app_specific_info.read,
            *args
        )

    def test_read_missing_application_data(self):
        """
        Test that an InvalidKmipEncoding error is raised during the decoding of
        an ApplicationSpecificInformation object with the application data is
        missing from the encoding.
        """
        app_specific_info = attributes.ApplicationSpecificInformation()

        self.assertIsNone(app_specific_info.application_data)

        args = (self.no_application_data_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The ApplicationSpecificInformation encoding is missing the "
            "ApplicationData field.",
            app_specific_info.read,
            *args
        )

    def test_write(self):
        """
        Test that an ApplicationSpecificInformation object can be written to a
        buffer.
        """
        app_specific_info = attributes.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        buff = utils.BytearrayStream()
        app_specific_info.write(buff)

        self.assertEqual(len(self.full_encoding), len(buff))
        self.assertEqual(str(self.full_encoding), str(buff))

    def test_write_missing_application_namespace(self):
        """
        Test that an InvalidField error is raised during the encoding of an
        ApplicationSpecificInformation object when the object is missing the
        application namespace field.
        """
        app_specific_info = attributes.ApplicationSpecificInformation(
            application_data="www.example.com"
        )

        buff = utils.BytearrayStream()
        args = (buff, )
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The ApplicationSpecificInformation object is missing the "
            "ApplicationNamespace field.",
            app_specific_info.write,
            *args
        )

    def test_write_missing_application_data(self):
        """
        Test that an InvalidField error is raised during the encoding of an
        ApplicationSpecificInformation object when the object is missing the
        application data field.
        """
        app_specific_info = attributes.ApplicationSpecificInformation(
            application_namespace="ssl"
        )

        buff = utils.BytearrayStream()
        args = (buff, )
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The ApplicationSpecificInformation object is missing the "
            "ApplicationData field.",
            app_specific_info.write,
            *args
        )

    def test_repr(self):
        """
        Test that repr can be applied to an ApplicationSpecificInformation
        object.
        """
        app_specific_info = attributes.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        args = [
            "application_namespace='ssl'",
            "application_data='www.example.com'"
        ]
        self.assertEqual(
            "ApplicationSpecificInformation({})".format(", ".join(args)),
            repr(app_specific_info)
        )

    def test_str(self):
        """
        Test that str can be applied to an ApplicationSpecificInformation
        object.
        """
        app_specific_info = attributes.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        args = [
            ("application_namespace", "ssl"),
            ("application_data", "www.example.com")
        ]
        value = "{}".format(
            ", ".join(['"{}": "{}"'.format(arg[0], arg[1]) for arg in args])
        )
        self.assertEqual(
            "{" + value + "}",
            str(app_specific_info)
        )

    def test_comparison(self):
        """
        Test that the equality/inequality operators return True/False when
        comparing two ApplicationSpecificInformation objects with the same
        data.
        """
        a = attributes.ApplicationSpecificInformation()
        b = attributes.ApplicationSpecificInformation()

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = attributes.ApplicationSpecificInformation(
            application_namespace="test_namespace",
            application_data="test_data"
        )
        b = attributes.ApplicationSpecificInformation(
            application_namespace="test_namespace",
            application_data="test_data"
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_comparison_on_different_application_namespaces(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two ApplicationSpecificInformation objects with different
        data.
        """
        a = attributes.ApplicationSpecificInformation(
            application_namespace="test_namespace_1"
        )
        b = attributes.ApplicationSpecificInformation(
            application_namespace="test_namespace_2"
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_application_data(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two ApplicationSpecificInformation objects with different
        data.
        """
        a = attributes.ApplicationSpecificInformation(
            application_data="test_data_1"
        )
        b = attributes.ApplicationSpecificInformation(
            application_data="test_data_2"
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_type_mismatch(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing an ApplicationSpecificInformation object to a
        non-ApplicationSpecificInformation object.
        """
        a = attributes.ApplicationSpecificInformation(
            application_namespace="test_namespace",
            application_data="test_data"
        )
        b = "invalid"

        self.assertFalse(a == b)
        self.assertFalse(b == a)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import TestCase

from kmip.core import attributes
from kmip.core import misc
from kmip.core import objects
from kmip.core import utils

from kmip.core.messages import payloads


class TestRekeyKeyPairRequestPayload(TestCase):

    def setUp(self):
        super(TestRekeyKeyPairRequestPayload, self).setUp()

        self.uuid = '00000000-0000-0000-0000-000000000000'
        self.private_key_uuid = attributes.PrivateKeyUniqueIdentifier(
            self.uuid)
        self.offset = misc.Offset(0)
        self.common_template_attribute = objects.CommonTemplateAttribute()
        self.private_key_template_attribute = \
            objects.PrivateKeyTemplateAttribute()
        self.public_key_template_attribute = \
            objects.PublicKeyTemplateAttribute()

        self.encoding_empty = utils.BytearrayStream((
            b'\x42\x00\x79\x01\x00\x00\x00\x00'))
        self.encoding_full = utils.BytearrayStream((
            b'\x42\x00\x79\x01\x00\x00\x00\x58\x42\x00\x66\x07\x00\x00\x00\x24'
            b'\x30\x30\x30\x30\x30\x30\x30\x30\x2d\x30\x30\x30\x30\x2d\x30\x30'
            b'\x30\x30\x2d\x30\x30\x30\x30\x2d\x30\x30\x30\x30\x30\x30\x30\x30'
            b'\x30\x30\x30\x30\x00\x00\x00\x00\x42\x00\x58\x0A\x00\x00\x00\x04'
            b'\x00\x00\x00\x00\x00\x00\x00\x00\x42\x00\x1F\x01\x00\x00\x00\x00'
            b'\x42\x00\x65\x01\x00\x00\x00\x00\x42\x00\x6E\x01\x00\x00\x00\x00'
            ))

    def tearDown(self):
        super(TestRekeyKeyPairRequestPayload, self).tearDown()

    def test_init_with_none(self):
        payloads.RekeyKeyPairRequestPayload()

    def test_init_with_args(self):
        payloads.RekeyKeyPairRequestPayload(
            self.private_key_uuid, self.offset, self.common_template_attribute,
            self.private_key_template_attribute,
            self.public_key_template_attribute)

    def test_validate_with_invalid_private_key_unique_identifier(self):
        kwargs = {'private_key_uuid': 'invalid', 'offset': None,
                  'common_template_attribute': None,
                  'private_key_template_attribute': None,
                  'public_key_template_attribute': None}
        self.assertRaisesRegex(
            TypeError, "invalid private key unique identifier",
            payloads.RekeyKeyPairRequestPayload, **kwargs)

    def test_validate_with_invalid_offset(self):
        kwargs = {'private_key_uuid': None, 'offset': 'invalid',
                  'common_template_attribute': None,
                  'private_key_template_attribute': None,
                  'public_key_template_attribute': None}
        self.assertRaisesRegex(
            TypeError, "invalid offset",
            payloads.RekeyKeyPairRequestPayload, **kwargs)

    def test_validate_with_invalid_common_template_attribute(self):
        kwargs = {'private_key_uuid': None, 'offset': None,
                  'common_template_attribute': 'invalid',
                  'private_key_template_attribute': None,
                  'public_key_template_attribute': None}
        self.assertRaisesRegex(
            TypeError, "invalid common template attribute",
            payloads.RekeyKeyPairRequestPayload, **kwargs)

    def test_validate_with_invalid_private_key_template_attribute(self):
        kwargs = {'private_key_uuid': None, 'offset': None,
                  'common_template_attribute': None,
                  'private_key_template_attribute': 'invalid',
                  'public_key_template_attribute': None}
        self.assertRaisesRegex(
            TypeError, "invalid private key template attribute",
            payloads.RekeyKeyPairRequestPayload, **kwargs)

    def test_validate_with_invalid_public_key_template_attribute(self):
        kwargs = {'private_key_uuid': None, 'offset': None,
                  'common_template_attribute': None,
                  'private_key_template_attribute': None,
                  'public_key_template_attribute': 'invalid'}
        self.assertRaisesRegex(
            TypeError, "invalid public key template attribute",
            payloads.RekeyKeyPairRequestPayload, **kwargs)

    def _test_read(self, stream, payload, private_key_uuid, offset,
                   common_template_attribute, private_key_template_attribute,
                   public_key_template_attribute):
        payload.read(stream)

        msg = "private_key_uuid decoding mismatch"
        msg += "; expected {0}, received {1}".format(
            private_key_uuid, payload.private_key_uuid)
        self.assertEqual(private_key_uuid, payload.private_key_uuid, msg)

        msg = "offset decoding mismatch"
        msg += "; expected {0}, received {1}".format(offset, payload.offset)
        self.assertEqual(offset, payload.offset, msg)

        msg = "common_template_attribute decoding mismatch"
        msg += "; expected {0}, received {1}".format(
            common_template_attribute, payload.common_template_attribute)
        self.assertEqual(common_template_attribute,
                         payload.common_template_attribute, msg)

        msg = "private_key_template_attribute decoding mismatch"
        msg += "; expected {0}, received {1}".format(
            private_key_template_attribute,
            payload.private_key_template_attribute)
        self.assertEqual(private_key_template_attribute,
                         payload.private_key_template_attribute, msg)

        msg = "public_key_template_attribute decoding mismatch"
        msg += "; expected {0}, received {1}".format(
            public_key_template_attribute,
            payload.public_key_template_attribute)
        self.assertEqual(public_key_template_attribute,
                         payload.public_key_template_attribute, msg)

    def test_read_with_none(self):
        stream = self.encoding_empty
        payload = payloads.RekeyKeyPairRequestPayload()

        self._test_read(stream, payload, None, None, None, None, None)

    def test_read_with_args(self):
        stream = self.encoding_full
        payload = payloads.RekeyKeyPairRequestPayload()

        self._test_read(stream, payload, self.private_key_uuid, self.offset,
                        self.common_template_attribute,
                        self.private_key_template_attribute,
                        self.public_key_template_attribute)

    def _test_write(self, stream, payload, expected):
        payload.write(stream)

        length_expected = len(expected)
        length_received = len(stream)

        msg = "encoding lengths not equal"
        msg += "; expected {0}, received {1}".format(
            length_expected, length_received)
        self.assertEqual(length_expected, length_received, msg)

        msg = "encoding mismatch"
        msg += ";\nexpected:\n{0}\nreceived:\n{1}".format(expected, stream)

        self.assertEqual(expected, stream, msg)

    def test_write_with_none(self):
        stream = utils.BytearrayStream()
        payload = payloads.RekeyKeyPairRequestPayload()

        self._test_write(stream, payload, self.encoding_empty)

    def test_write_with_args(self):
        stream = utils.BytearrayStream()
        payload = payloads.RekeyKeyPairRequestPayload(
            self.private_key_uuid, self.offset, self.common_template_attribute,
            self.private_key_template_attribute,
            self.public_key_template_attribute)

        self._test_write(stream, payload, self.encoding_full)


# TODO (ph) Replicate CreateKeyPairResponsePayload test suite here
class TestRekeyKeyPairResponsePayload(TestCase):

    def setUp(self):
        super(TestRekeyKeyPairResponsePayload, self).setUp()

        self.encoding_empty = utils.BytearrayStream((
            b'\x42\x00\x7C\x01\x00\x00\x00\x10\x42\x00\x66\x07\x00\x00\x00\x00'
            b'\x42\x00\x6F\x07\x00\x00\x00\x00'))
        self.encoding_full = utils.BytearrayStream((
            b'\x42\x00\x7C\x01\x00\x00\x00\x70\x42\x00\x66\x07\x00\x00\x00\x24'
            b'\x30\x30\x30\x30\x30\x30\x30\x30\x2d\x30\x30\x30\x30\x2d\x30\x30'
            b'\x30\x30\x2d\x30\x30\x30\x30\x2d\x30\x30\x30\x30\x30\x30\x30\x30'
            b'\x30\x30\x30\x30\x00\x00\x00\x00\x42\x00\x6F\x07\x00\x00\x00\x24'
            b'\x30\x30\x30\x30\x30\x30\x30\x30\x2d\x30\x30\x30\x30\x2d\x30\x30'
            b'\x30\x30\x2d\x30\x30\x30\x30\x2d\x30\x30\x30\x30\x30\x30\x30\x30'
            b'\x30\x30\x30\x30\x00\x00\x00\x00\x42\x00\x65\x01\x00\x00\x00\x00'
            b'\x42\x00\x6E\x01\x00\x00\x00\x00'))

    def tearDown(self):
        super(TestRekeyKeyPairResponsePayload, self).tearDown()



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from testtools import TestCase

from kmip.pie.objects import Key


class DummyKey(Key):
    """
    A dummy Key subclass for testing purposes.
    """

    def __init__(self):
        """
        Create a DummyKey
        """
        super(DummyKey, self).__init__()

    def validate(self):
        return

    def __repr__(self):
        return ''

    def __str__(self):
        return ''

    def __eq__(self, other):
        return True

    def __ne__(self, other):
        return False


class TestKey(TestCase):
    """
    Test suite for Key.

    Since Key is an ABC abstract class, all tests are run against a dummy
    subclass defined above, DummyKey.
    """

    def setUp(self):
        super(TestKey, self).setUp()

    def tearDown(self):
        super(TestKey, self).tearDown()

    def test_init(self):
        """
        Test that a complete subclass of Key can be
        instantiated.
        """
        DummyKey()

    def test_repr(self):
        """
        Test that repr can be applied to a Key.
        """
        dummy = DummyKey()
        repr(dummy)

    def test_str(self):
        """
        Test that str can be applied to a Key.
        """
        dummy = DummyKey()
        str(dummy)

    def test_eq(self):
        """
        Test that equality can be applied to a Key.
        """
        dummy = DummyKey()
        self.assertTrue(dummy == dummy)

    def test_ne(self):
        """
        Test that inequality can be applied to a Key.
        """
        dummy = DummyKey()
        self.assertFalse(dummy != dummy)

    def test_key_wrapping_data_invalid(self):
        """
        Test that the right error is raised when setting the key wrapping
        data with an invalid value.
        """
        dummy = DummyKey()
        args = (dummy, 'key_wrapping_data', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Key wrapping data must be a dictionary.",
            setattr,
            *args
        )



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core.enums import CredentialType
from kmip.core.enums import KeyFormatType as KeyFormatTypeEnum
from kmip.core.enums import Operation
from kmip.core.enums import ResultStatus

from kmip.core.factories.attributes import AttributeFactory
from kmip.core.factories.credentials import CredentialFactory

from kmip.core.misc import KeyFormatType

from kmip.demos import utils

from kmip.services.kmip_client import KMIPProxy

import logging
import sys


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    # Build and parse arguments
    parser = utils.build_cli_parser(Operation.GET)
    opts, args = parser.parse_args(sys.argv[1:])

    username = opts.username
    password = opts.password
    config = opts.config
    uuid = opts.uuid
    format_type = opts.format

    # Exit early if the UUID is not specified
    if uuid is None:
        logger.error('No UUID provided, exiting early from demo')
        sys.exit()

    format_type_enum = None
    if format_type is not None:
        format_type_enum = getattr(KeyFormatTypeEnum, format_type, None)

        if format_type_enum is None:
            logger.error(
                "Invalid key format type specified; exiting early from demo")
            sys.exit()

    attribute_factory = AttributeFactory()
    credential_factory = CredentialFactory()

    # Build the KMIP server account credentials
    # TODO (peter-hamilton) Move up into KMIPProxy
    if (username is None) and (password is None):
        credential = None
    else:
        credential_type = CredentialType.USERNAME_AND_PASSWORD
        credential_value = {'Username': username,
                            'Password': password}
        credential = credential_factory.create_credential(credential_type,
                                                          credential_value)

    key_format_type = None
    if format_type_enum is not None:
        key_format_type = KeyFormatType(format_type_enum)

    # Build the client and connect to the server
    client = KMIPProxy(config=config, config_file=opts.config_file)
    client.open()

    # Retrieve the SYMMETRIC_KEY object
    result = client.get(uuid=uuid, key_format_type=key_format_type,
                        credential=credential)
    client.close()

    # Display operation results
    logger.info('get() result status: {0}'.format(
        result.result_status.value))

    if result.result_status.value == ResultStatus.SUCCESS:
        logger.info('retrieved object type: {0}'.format(
            result.object_type.value))
        logger.info('retrieved UUID: {0}'.format(result.uuid))

        utils.log_secret(logger, result.object_type, result.secret)
    else:
        logger.info('get() result reason: {0}'.format(
            result.result_reason.value))
        logger.info('get() result message: {0}'.format(
            result.result_message.value))
